@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -55,48 +55,9 @@
 #include "gstpriv.h"
 #include "re.h"
 
-#include "../libffi/include/ffi.h"
+#include "ffi.h"
 #include <ltdl.h>
 
-typedef enum
-{				/* types for C parameters */
-  CDATA_UNKNOWN,		/* when there is no type a priori */
-  CDATA_CHAR,
-  CDATA_STRING,
-  CDATA_STRING_OUT,		/* for things that modify string params */
-  CDATA_SYMBOL,
-  CDATA_BYTEARRAY,
-  CDATA_BYTEARRAY_OUT,
-  CDATA_BOOLEAN,
-  CDATA_INT,
-  CDATA_UINT,
-  CDATA_LONG,
-  CDATA_ULONG,
-  CDATA_FLOAT,
-  CDATA_DOUBLE,
-  CDATA_LONG_DOUBLE,
-  CDATA_VOID,			/* valid only as a return type */
-  CDATA_VARIADIC,		/* for parameters, this param is an
-				   array to be interpreted as
-				   arguments.  Note that only simple
-				   conversions are performed in this
-				   case.  */
-  CDATA_VARIADIC_OOP,		/* for parameters, this param is an
-				   array whose elements are OOPs to be
-				   passed.  */
-  CDATA_COBJECT,		/* a C object is being passed */
-  CDATA_COBJECT_PTR,		/* a C object pointer is being passed */
-  CDATA_OOP,			/* no conversion to-from C (OOP) */
-  CDATA_SELF,			/* pass self as the corresponding
-				   argument */
-  CDATA_SELF_OOP,		/* pass self as an OOP */
-  CDATA_WCHAR,
-  CDATA_WSTRING,
-  CDATA_WSTRING_OUT,
-  CDATA_SYMBOL_OUT
-}
-cdata_type;
-
 typedef struct symbol_type_map
 {
   OOP *symbol;
@@ -144,15 +105,25 @@ typedef struct cfunc_info
 }
 cfunc_info;
 
-typedef struct gst_stat
+struct gst_stat_struct
 {
   unsigned short st_mode;	/* protection */
   long st_size;			/* total size, in bytes */
   long st_aTime;		/* time of last access */
   long st_mTime;		/* time of last modification */
   long st_cTime;		/* time of last change */
+};
+
+typedef struct gst_stat
+{
+  OBJ_HEADER;
+  OOP st_mode;		/* protection */
+  OOP st_size;		/* total size, in bytes */
+  OOP st_aTime;		/* time of last access */
+  OOP st_mTime;		/* time of last modification */
+  OOP st_cTime;		/* time of last change */
 }
-gst_stat;
+*gst_stat;
 
 
 
@@ -207,15 +178,19 @@ static OOP classify_type_symbol (OOP symbolOOP,
 static int get_errno (void);
 
 /* Encapsulate binary incompatibilities between various C libraries.  */
-static int my_utime (const char *name,
-		     long new_atime,
-		     long new_mtime);
+static int my_stat_old (const char *name,
+		        struct gst_stat_struct * out);
+static int my_lstat_old (const char *name,
+		         struct gst_stat_struct * out);
 static int my_stat (const char *name,
-		    gst_stat * out);
+		    OOP out);
 static int my_lstat (const char *name,
-		     gst_stat * out);
+		     OOP out);
 static int my_putenv (const char *str);
 static int my_chdir (const char *str);
+static int my_symlink (const char* oldpath, const char* newpath);
+static char *my_mkdtemp (char* template);
+static int my_mkdir (const char* name, int mode);
 static DIR *my_opendir (const char *str);
 static char *extract_dirent_name (struct dirent *dir);
 
@@ -231,38 +206,46 @@ static cfunc_info *c_func_cur = NULL;
 
 /* printable names for corresponding C types */
 static const char *c_type_name[] = {
-  "void?",			/* CDATA_UNKNOWN */
   "char",			/* CDATA_CHAR */
+  "unsigned char",		/* CDATA_UCHAR */
+  "short",			/* CDATA_SHORT */
+  "unsigned short",		/* CDATA_USHORT */
+  "long",			/* CDATA_LONG */
+  "unsigned long",		/* CDATA_ULONG */
+  "float",			/* CDATA_FLOAT */
+  "double",			/* CDATA_DOUBLE */
   "char *",			/* CDATA_STRING */
+  "OOP",			/* CDATA_OOP */
+  "int",			/* CDATA_INT */
+  "unsigned int",		/* CDATA_UINT */
+  "long double",		/* CDATA_LONG_DOUBLE */
+
+  "void?",			/* CDATA_UNKNOWN */
   "char *",			/* CDATA_STRING_OUT */
   "char *",			/* CDATA_SYMBOL */
   "char *",			/* CDATA_BYTEARRAY */
   "char *",			/* CDATA_BYTEARRAY_OUT */
   "int",			/* CDATA_BOOLEAN */
-  "int",			/* CDATA_INT */
-  "unsigned int",		/* CDATA_UINT */
-  "long",			/* CDATA_LONG */
-  "unsigned long",		/* CDATA_ULONG */
-  "float",			/* CDATA_FLOAT */
-  "double",			/* CDATA_DOUBLE */
-  "long double",		/* CDATA_LONG_DOUBLE */
   "void?",			/* CDATA_VOID */
   "...",			/* CDATA_VARIADIC */
   "...",			/* CDATA_VARIADIC_OOP */
   "void *",			/* CDATA_COBJECT -- this is misleading */
   "void **",			/* CDATA_COBJECT_PTR */
-  "OOP",			/* CDATA_OOP */
   "void?",			/* CDATA_SELF */
   "OOP",			/* CDATA_SELF_OOP */
   "wchar_t",			/* CDATA_WCHAR */
   "wchar_t *",			/* CDATA_WSTRING */
   "wchar_t *",			/* CDATA_WSTRING_OUT */
+  "char *",			/* CDATA_SYMBOL_OUT */
 };
 
 /* A map between symbols and the cdata_type enum.  */
 static const symbol_type_map type_map[] = {
   {&_gst_unknown_symbol, CDATA_UNKNOWN},
   {&_gst_char_symbol, CDATA_CHAR},
+  {&_gst_uchar_symbol, CDATA_UCHAR},
+  {&_gst_short_symbol, CDATA_SHORT},
+  {&_gst_ushort_symbol, CDATA_USHORT},
   {&_gst_string_symbol, CDATA_STRING},
   {&_gst_string_out_symbol, CDATA_STRING_OUT},
   {&_gst_symbol_symbol, CDATA_SYMBOL},
@@ -315,28 +298,18 @@ get_errno (void)
   return (old);
 }
 
-int
-my_utime (const char *name,
-	  long new_atime,
-	  long new_mtime)
+static inline int
+adjust_time (time_t t)
 {
-  struct timeval times[2];
-  int result;
-  times[0].tv_sec = new_atime + 86400 * 10957;
-  times[1].tv_sec = new_mtime + 86400 * 10957;
-  times[0].tv_usec = times[1].tv_usec = 0;
-  result = utimes (name, times);
-  if (!result)
-    errno = 0;
-  return (result);
+  return _gst_adjust_time_zone (t) - 86400 * 10957;
 }
 
-int
-my_stat (const char *name,
-	 gst_stat * out)
+static inline int
+my_stat_old (const char *name,
+	     struct gst_stat_struct * out)
 {
   int result;
-  static struct stat statOut;
+  struct stat statOut;
 
   result = stat (name, &statOut);
   if (!result)
@@ -344,20 +317,41 @@ my_stat (const char *name,
       errno = 0;
       out->st_mode = statOut.st_mode;
       out->st_size = statOut.st_size;
-      out->st_aTime = _gst_adjust_time_zone (statOut.st_atime) - 86400 * 10957;
-      out->st_mTime = _gst_adjust_time_zone (statOut.st_mtime) - 86400 * 10957;
-      out->st_cTime = _gst_adjust_time_zone (statOut.st_ctime) - 86400 * 10957;
+      out->st_aTime = adjust_time (statOut.st_atime);
+      out->st_mTime = adjust_time (statOut.st_mtime);
+      out->st_cTime = adjust_time (statOut.st_ctime);
+    }
+  return (result);
+}
+
+int
+my_stat (const char *name,
+	 OOP out)
+{
+  int result;
+  struct stat statOut;
+
+  result = stat (name, &statOut);
+  if (!result)
+    {
+      gst_stat obj = (gst_stat) OOP_TO_OBJ (out);
+      errno = 0;
+      obj->st_mode = FROM_INT (statOut.st_mode);
+      obj->st_aTime = FROM_INT (adjust_time (statOut.st_atime));
+      obj->st_mTime = FROM_INT (adjust_time (statOut.st_mtime));
+      obj->st_cTime = FROM_INT (adjust_time (statOut.st_ctime));
+      obj->st_size = FROM_OFF_T (statOut.st_size);
     }
   return (result);
 }
 
 #ifdef HAVE_LSTAT
-int
-my_lstat (const char *name,
-	 gst_stat * out)
+static inline int
+my_lstat_old (const char *name,
+	      struct gst_stat_struct * out)
 {
   int result;
-  static struct stat statOut;
+  struct stat statOut;
 
   result = lstat (name, &statOut);
   if (!result)
@@ -365,14 +359,36 @@ my_lstat (const char *name,
       errno = 0;
       out->st_mode = statOut.st_mode;
       out->st_size = statOut.st_size;
-      out->st_aTime = _gst_adjust_time_zone (statOut.st_atime) - 86400 * 10957;
-      out->st_mTime = _gst_adjust_time_zone (statOut.st_mtime) - 86400 * 10957;
-      out->st_cTime = _gst_adjust_time_zone (statOut.st_ctime) - 86400 * 10957;
+      out->st_aTime = adjust_time (statOut.st_atime);
+      out->st_mTime = adjust_time (statOut.st_mtime);
+      out->st_cTime = adjust_time (statOut.st_ctime);
+    }
+  return (result);
+}
+
+int
+my_lstat (const char *name,
+	 OOP out)
+{
+  int result;
+  struct stat statOut;
+
+  result = lstat (name, &statOut);
+  if (!result)
+    {
+      gst_stat obj = (gst_stat) OOP_TO_OBJ (out);
+      errno = 0;
+      obj->st_mode = FROM_INT (statOut.st_mode);
+      obj->st_aTime = FROM_INT (adjust_time (statOut.st_atime));
+      obj->st_mTime = FROM_INT (adjust_time (statOut.st_mtime));
+      obj->st_cTime = FROM_INT (adjust_time (statOut.st_ctime));
+      obj->st_size = FROM_OFF_T (statOut.st_size);
     }
   return (result);
 }
 #else
 #define my_lstat my_stat
+#define my_lstat_old my_stat_old
 #endif
 
 int
@@ -400,6 +416,21 @@ my_chdir (const char *dir)
   return (status);
 }
 
+static int 
+my_mkdir (const char* name,
+	  int mode)
+{
+  int retstat;
+#ifdef __MSVCRT__
+  retstat = mkdir (name);
+  if (retstat == 0)
+    retstat = chmod (name, mode);
+#else
+  retstat = mkdir (name, mode);
+#endif
+  return retstat;
+}
+
 DIR *
 my_opendir (const char *dir)
 {
@@ -455,7 +486,7 @@ get_argc (void)
 const char *
 get_argv (int n)
 {
-  return (n <= _gst_smalltalk_passed_argc
+  return (n >= 1 && n <= _gst_smalltalk_passed_argc
 	  ? _gst_smalltalk_passed_argv[n - 1]
 	  : NULL);
 }
@@ -465,14 +496,14 @@ dld_open (const char *filename)
 {
 #ifdef ENABLE_DLD
   lt_dlhandle handle;
-  void (*initModule) ();
+  void (*initModule) (struct VMProxy *);
 
   handle = lt_dlopenext (filename);
   if (handle)
     {
       initModule = lt_dlsym (handle, "gst_initModule");
       if (initModule)
-	initModule (&gst_interpreter_proxy);
+	initModule (_gst_get_vmproxy ());
     }
 
   return (handle);
@@ -488,7 +519,11 @@ init_dld (void)
 {
   char *modules;
   lt_dlinit ();
-  lt_dladdsearchdir (MODULE_PATH);
+
+  modules = _gst_relocate_path (MODULE_PATH);
+  lt_dladdsearchdir (modules);
+  free (modules);
+
   if ((modules = getenv ("SMALLTALK_MODULES")))
     lt_dladdsearchdir (modules);
 
@@ -521,9 +556,12 @@ _gst_init_cfuncs (void)
 
   _gst_define_cfunc ("errno", get_errno);
   _gst_define_cfunc ("strerror", strerror);
-  _gst_define_cfunc ("stat", my_stat);
-  _gst_define_cfunc ("lstat", my_lstat);
-  _gst_define_cfunc ("utime", my_utime);
+  _gst_define_cfunc ("stat", my_stat_old);
+  _gst_define_cfunc ("lstat", my_lstat_old);
+  _gst_define_cfunc ("stat_obj", my_stat);
+  _gst_define_cfunc ("lstat_obj", my_lstat);
+  _gst_define_cfunc ("utime", _gst_set_file_access_times);
+  _gst_define_cfunc ("chmod", chmod);
 
   _gst_define_cfunc ("opendir", my_opendir);
   _gst_define_cfunc ("closedir", closedir);
@@ -531,11 +569,13 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("rewinddir", rewinddir);
   _gst_define_cfunc ("extractDirentName", extract_dirent_name);
 
+  _gst_define_cfunc ("symlink", my_symlink);
   _gst_define_cfunc ("unlink", unlink);
   _gst_define_cfunc ("rename", rename);
   _gst_define_cfunc ("rmdir", rmdir);
   _gst_define_cfunc ("chdir", my_chdir);
-  _gst_define_cfunc ("mkdir", mkdir);
+  _gst_define_cfunc ("mkdir", my_mkdir);
+  _gst_define_cfunc ("mkdtemp", my_mkdtemp);
   _gst_define_cfunc ("getCurDirName", _gst_get_cur_dir_name);
 
   _gst_define_cfunc ("fileIsReadable", _gst_file_is_readable);
@@ -612,6 +652,63 @@ lookup_function (const char *funcName)
 }
 
 
+int
+_gst_c_type_size (int type)
+{
+  switch (type)
+    {
+    case CDATA_CHAR:
+      return sizeof (char);
+    case CDATA_UCHAR:
+      return sizeof (unsigned char);
+
+    case CDATA_SHORT:
+      return sizeof (short);
+    case CDATA_USHORT:
+      return sizeof (unsigned short);
+
+    case CDATA_INT:
+      return sizeof (int);
+    case CDATA_UINT:
+      return sizeof (unsigned int);
+
+    case CDATA_LONG:
+      return sizeof (long);
+    case CDATA_ULONG:
+      return sizeof (unsigned long);
+
+    case CDATA_FLOAT:
+      return sizeof (float);
+    case CDATA_DOUBLE:
+      return sizeof (double);
+    case CDATA_LONG_DOUBLE:
+      return sizeof (long double);
+
+    case CDATA_OOP:
+      return sizeof (OOP);
+
+    case CDATA_WCHAR:
+      return sizeof (wchar_t);
+
+    case CDATA_WSTRING:
+      return sizeof (wchar_t *);
+
+    case CDATA_STRING:
+    case CDATA_STRING_OUT:
+    case CDATA_SYMBOL:
+    case CDATA_BYTEARRAY:
+    case CDATA_BYTEARRAY_OUT:
+    case CDATA_SYMBOL_OUT:
+      return sizeof (char *);
+
+    case CDATA_COBJECT:
+      return sizeof (void *);
+
+    case CDATA_COBJECT_PTR:
+      return sizeof (void **);
+    }
+}
+
 OOP
 _gst_invoke_croutine (OOP cFuncOOP,
 		      OOP receiver,
@@ -631,11 +728,11 @@ _gst_invoke_croutine (OOP cFuncOOP,
   if (IS_NIL (desc->cFunction))
     return (NULL);
 
-  c_func_cur = (cfunc_info *) COBJECT_VALUE (desc->cFunction);
+  c_func_cur = (cfunc_info *) cobject_value (desc->cFunction);
   if (!c_func_cur)
     return (NULL);
 
-  fixedArgs = TO_INT (desc->numFixedArgs);
+  fixedArgs = NUM_INDEXABLE_FIELDS (cFuncOOP);
   totalArgs = 0;
   for (si = i = 0; i < fixedArgs; i++)
     {
@@ -712,7 +809,7 @@ _gst_invoke_croutine (OOP cFuncOOP,
         switch (arg->cType)
           {
           case CDATA_COBJECT_PTR:
-	    SET_COBJECT_VALUE (arg->oop, arg->u.cObjectPtrVal.ptrVal);
+	    set_cobject_value (arg->oop, arg->u.cObjectPtrVal.ptrVal);
 	    continue;
 
           case CDATA_WSTRING_OUT:
@@ -770,12 +867,17 @@ get_ffi_type (OOP returnTypeOOP)
 
     case CDATA_VOID:
     case CDATA_INT:
-    case CDATA_UINT:
     case CDATA_CHAR:
+    case CDATA_SHORT:
     case CDATA_WCHAR:
     case CDATA_BOOLEAN:
       return &ffi_type_sint;
 
+    case CDATA_UINT:
+    case CDATA_UCHAR:
+    case CDATA_USHORT:
+      return &ffi_type_uint;
+
    case CDATA_FLOAT:
      return &ffi_type_float;
 
@@ -876,11 +978,30 @@ push_smalltalk_obj (OOP oop,
 
         case CDATA_INT:
 	case CDATA_UINT:
-	case CDATA_CHAR:
 	  cp->u.intVal = TO_C_INT (oop);
 	  SET_TYPE (&ffi_type_sint);
 	  return;
 
+	case CDATA_CHAR:
+	  cp->u.intVal = (char) TO_C_INT (oop);
+	  SET_TYPE (&ffi_type_sint);
+	  return;
+
+	case CDATA_UCHAR:
+	  cp->u.intVal = (unsigned char) TO_C_INT (oop);
+	  SET_TYPE (&ffi_type_sint);
+	  return;
+
+	case CDATA_SHORT:
+	  cp->u.intVal = (short) TO_C_INT (oop);
+	  SET_TYPE (&ffi_type_sint);
+	  return;
+
+	case CDATA_USHORT:
+	  cp->u.intVal = (unsigned short) TO_C_INT (oop);
+	  SET_TYPE (&ffi_type_sint);
+	  return;
+
 	case CDATA_DOUBLE:
           cp->u.doubleVal = (double) TO_C_LONG (oop);
 	  SET_TYPE (&ffi_type_double);
@@ -910,6 +1031,9 @@ push_smalltalk_obj (OOP oop,
         case CDATA_INT:
 	case CDATA_UINT:
 	case CDATA_CHAR:
+	case CDATA_UCHAR:
+	case CDATA_SHORT:
+	case CDATA_USHORT:
 	case CDATA_BOOLEAN:
 	  cp->u.intVal = (oop == _gst_true_oop);
 	  SET_TYPE (&ffi_type_sint);
@@ -918,7 +1042,7 @@ push_smalltalk_obj (OOP oop,
     }
 
   else if ((class == _gst_char_class
-	    && (cType == CDATA_CHAR || cType == CDATA_WCHAR))
+	    && (cType == CDATA_CHAR || cType == CDATA_UCHAR || cType == CDATA_WCHAR))
            || (class == _gst_unicode_character_class && cType == CDATA_WCHAR))
     {
       cp->u.intVal = CHAR_OOP_VALUE (oop);
@@ -986,13 +1110,13 @@ push_smalltalk_obj (OOP oop,
 	  /* Set up an indirect pointer to protect against the OOP
 	     moving during the call-out.  */
 	  cp->u.cObjectPtrVal.pPtrVal = &cp->u.cObjectPtrVal.ptrVal;
-	  cp->u.cObjectPtrVal.ptrVal = COBJECT_VALUE (oop);
+	  cp->u.cObjectPtrVal.ptrVal = cobject_value (oop);
 	  cp->oop = oop;
 	  SET_TYPE (&ffi_type_pointer);
 	  return;
 
 	case CDATA_COBJECT:
-	  cp->u.ptrVal = COBJECT_VALUE (oop);
+	  cp->u.ptrVal = cobject_value (oop);
 	  SET_TYPE (&ffi_type_pointer);
 	  return;
 	}
@@ -1036,6 +1160,7 @@ c_to_smalltalk (cparam *result, OOP returnTypeOOP)
       break;
 
     case CDATA_CHAR:
+    case CDATA_UCHAR:
       resultOOP = CHAR_OOP_AT ((gst_uchar) result->u.intVal);
       break;
 
@@ -1048,11 +1173,19 @@ c_to_smalltalk (cparam *result, OOP returnTypeOOP)
       break;
 
     case CDATA_INT:
-      resultOOP = FROM_C_INT ((long) result->u.intVal);
+      resultOOP = FROM_C_INT ((int) result->u.intVal);
       break;
 
     case CDATA_UINT:
-      resultOOP = FROM_C_INT ((long) result->u.intVal);
+      resultOOP = FROM_C_UINT ((unsigned int) result->u.intVal);
+      break;
+
+    case CDATA_SHORT:
+      resultOOP = FROM_INT ((short) result->u.intVal);
+      break;
+
+    case CDATA_USHORT:
+      resultOOP = FROM_INT ((unsigned short) result->u.intVal);
       break;
 
     case CDATA_LONG:
@@ -1085,9 +1218,9 @@ c_to_smalltalk (cparam *result, OOP returnTypeOOP)
       else if (returnType == CDATA_COBJECT)
 	{
 	  if (IS_INT (returnTypeOOP))
-	    resultOOP = COBJECT_NEW (result->u.ptrVal);
-	  else
-	    resultOOP = _gst_c_object_new_typed (result->u.ptrVal, returnTypeOOP);
+	    returnTypeOOP = _gst_nil_oop;
+	  resultOOP = COBJECT_NEW (result->u.ptrVal, returnTypeOOP,
+				   _gst_c_object_class);
 	}
       else if (returnType == CDATA_STRING || returnType == CDATA_STRING_OUT)
 	{
@@ -1142,7 +1275,8 @@ bad_type (OOP class_oop,
 
 
 OOP
-_gst_make_descriptor (OOP funcNameOOP,
+_gst_make_descriptor (OOP classOOP,
+		      OOP funcNameOOP,
 		      OOP returnTypeOOP,
 		      OOP argsOOP)
 {
@@ -1167,27 +1301,38 @@ _gst_make_descriptor (OOP funcNameOOP,
      OOPs */
   incPtr = INC_SAVE_POINTER ();
 
-  cFunction = COBJECT_NEW (cfi);
+  cFunction = COBJECT_NEW (cfi, _gst_nil_oop, _gst_c_object_class);
   INC_ADD_OOP (cFunction);
 
   cFunctionName = _gst_string_new (funcName);
+  xfree (funcName);
   INC_ADD_OOP (cFunctionName);
 
-  desc = (gst_cfunc_descriptor)
-    new_instance_with (_gst_c_func_descriptor_class, numArgs,
-		       &descOOP);
+  desc = (gst_cfunc_descriptor) new_instance_with (classOOP, numArgs, &descOOP);
 
   desc->cFunction = cFunction;
   desc->cFunctionName = cFunctionName;
-  desc->numFixedArgs = FROM_INT (numArgs);
+  desc->tagOOP = _gst_nil_oop;
   desc->returnType = classify_type_symbol (returnTypeOOP, true);
+  if (desc->returnType == _gst_nil_oop)
+    goto error;
+
   for (i = 1; i <= numArgs; i++)
-    desc->argTypes[i - 1] =
-      classify_type_symbol (ARRAY_AT (argsOOP, i), false);
+    {
+      OOP type;
+      type = desc->argTypes[i - 1] =
+        classify_type_symbol (ARRAY_AT (argsOOP, i), false);
+      if (type == _gst_nil_oop)
+	goto error;
+    }
 
-  xfree (funcName);
   INC_RESTORE_POINTER (incPtr);
   return (descOOP);
+
+ error:
+  INC_RESTORE_POINTER (incPtr);
+  return (NULL);
+
 }
 
 OOP
@@ -1195,8 +1340,6 @@ classify_type_symbol (OOP symbolOOP,
 		      mst_Boolean isReturn)
 {
   const symbol_type_map *sp;
-  char *symbolName;
-
   for (sp = type_map; sp->symbol != NULL; sp++)
     {
       if (*sp->symbol == symbolOOP)
@@ -1209,14 +1352,7 @@ classify_type_symbol (OOP symbolOOP,
 	return (symbolOOP);	/* this is the type we want! */
     }
 
-  symbolName = _gst_to_cstring (symbolOOP);	/* yeah, yeah...but
-						   they have the same
-						   representation! */
-  _gst_errorf ("Unknown data type symbol: %s", symbolName);
-
-  xfree (symbolName);
-
-  return (FROM_INT (CDATA_UNKNOWN));
+  return _gst_nil_oop;
 }
 
 void
@@ -1232,3 +1368,19 @@ _gst_set_errno(int errnum)
   _gst_errno = errnum;
 #endif
 }
+
+
+/* TODO: check if this can be changed to an extern declaration and/or
+   an AC_CHECK_DECLS test.  */
+
+int
+my_symlink (const char* oldpath, const char* newpath)
+{
+  return symlink (oldpath,  newpath);
+}
+ 
+char*
+my_mkdtemp(char* template)
+{
+  return mkdtemp(template);
+}
