@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -72,15 +72,6 @@
 #define LITERAL_VEC_CHUNK_SIZE		32
 
 
-typedef struct gst_file_segment
-{
-  OBJ_HEADER;
-  OOP fileName;
-  OOP startPos;
-  OOP length;
-}
- *gst_file_segment;
-
 typedef struct method_attributes
 {
   struct method_attributes *next;
@@ -122,12 +113,6 @@ mst_Boolean _gst_skip_compilation = false;
    Most often, the caller does not care about the returned value,
    since it often is called from a radically different context.  */
 OOP _gst_last_returned_value = NULL;
-
-/* This is set to true by the parser or the compiler if an error
-   (respectively, a parse error or a semantic error) is found, and
-   avoids that _gst_execute_statements tries to execute the result of
-   the compilation.  */
-mst_Boolean _gst_had_error = false;
 
 
 
@@ -215,12 +200,6 @@ static mst_Boolean compile_statements (tree_node statementList,
    them into a symbol OOP and returns that symbol.  */
 static OOP compute_selector (tree_node selectorExpr);
 
-/* Given CONSTEXPR, a section of the syntax tree that represents a
-   Smalltalk constant, this routine creates and returns an OOP to be
-   stored as a method literal in the method that's currently being
-   compiled.  */
-static OOP make_constant_oop (tree_node constExpr);
-
 /* Creates a new Array object that contains the literals for the
    method that's being compiled and returns it.  As a side effect, the
    currently allocated working literal vector is freed.  If there were
@@ -254,20 +233,6 @@ static OOP method_info_new (OOP class,
 			    OOP sourceCode,
 			    OOP categoryOOP);
 
-/* Returns a FileSegment instance for the currently open compilation
-   stream.  FileSegment instances are used record information useful
-   in obtaining the source code for a method that's been compiled.
-   Depending on whether the input stream is a string or a FileStream,
-   the instance variables are different; for a string, the entire
-   contents of the string is preserved as the source code for the
-   method; for a disk file loaded from within the kernel source
-   directory, the file name, byte offset and length are preserved.  
-   The two parameters are used in the latter case and they represent
-   respectively the first byte of the method and the first byte after
-   the method.  Otherwise they should be -1.  */
-static OOP file_segment_new (int64_t startPos,
-			     int64_t endPos);
-
 /* This creates a CompiledBlock for the given BYTECODES.  The bytecodes
    are passed through the peephole optimizer and stored, the header is
    filled according to the given number of arguments ARGS and
@@ -501,12 +466,12 @@ _gst_install_initial_methods (void)
   install_method (termination_method);
 
   methodsForString = "\
-methodsFor: aCategoryString \
+methodsFor: aCategoryString [\
     \"Calling this method prepares the parser to receive methods \
       to be compiled and installed in the receiver's method dictionary. \
       The methods are put in the category identified by the parameter.\" \
     <primitive: VMpr_Behavior_methodsFor> \
-";
+]";
   _gst_set_compilation_class (_gst_behavior_class);
   _gst_set_compilation_category (_gst_string_new ("compiling methods"));
   _gst_push_smalltalk_string (_gst_string_new (methodsForString));
@@ -529,15 +494,54 @@ _gst_get_termination_method (void)
   return (termination_method);
 }
 
+static void
+invoke_hook_smalltalk (enum gst_vm_hook hook)
+{
+  const char *hook_name;
+  if (!_gst_kernel_initialized)
+    return;
+
+  switch (hook) {
+  case GST_BEFORE_EVAL:
+    hook_name = "beforeEvaluation";
+    break;
+
+  case GST_AFTER_EVAL:
+    hook_name = "afterEvaluation";
+    break;
+
+  case GST_RETURN_FROM_SNAPSHOT:
+    hook_name = "returnFromSnapshot";
+    break;
+
+  case GST_ABOUT_TO_QUIT:
+    hook_name = "aboutToQuit";
+    break;
+
+  case GST_ABOUT_TO_SNAPSHOT:
+    hook_name = "aboutToSnapshot";
+    break;
+
+  case GST_FINISHED_SNAPSHOT:
+    hook_name = "finishedSnapshot";
+    break;
+
+  default:
+    return;
+  }
+
+  _gst_msg_sendf (NULL, "%v %o changed: %S", 
+		  _gst_object_memory_class, hook_name);
+}
+
 void
-_gst_invoke_hook (const char *hook)
+_gst_invoke_hook (enum gst_vm_hook hook)
 {
   int save_execution;
   save_execution = _gst_execution_tracing;
   if (_gst_execution_tracing == 1)
     _gst_execution_tracing = 0;
-  _gst_msg_sendf (NULL, "%v %o changed: %S", 
-		  _gst_object_memory_class, hook);
+  invoke_hook_smalltalk (hook);
   _gst_execution_tracing = save_execution;
 }
 
@@ -652,7 +656,7 @@ _gst_execute_statements (tree_node temps,
   methodOOP =
     _gst_compile_method (_gst_make_method (&statements->location, &loc,
 					   messagePattern, temps, NULL,
-					   statements),
+					   statements, false),
 			 true, false);
 
   SET_CLASS_ENVIRONMENT (_gst_undefined_object_class,
@@ -679,8 +683,7 @@ _gst_execute_statements (tree_node temps,
   getrusage (RUSAGE_SELF, &startRusage);
 #endif
 
-  if (_gst_kernel_initialized)
-    _gst_invoke_hook ("beforeEvaluation");
+  _gst_invoke_hook (GST_BEFORE_EVAL);
 
   /* send a message to NIL, which will find this synthetic method
      definition in Object and execute it */
@@ -692,8 +695,7 @@ _gst_execute_statements (tree_node temps,
   getrusage (RUSAGE_SELF, &endRusage);
 #endif
 
-  if (_gst_kernel_initialized)
-    _gst_invoke_hook ("afterEvaluation");
+  _gst_invoke_hook (GST_AFTER_EVAL);
 
   INC_RESTORE_POINTER (incPtr);
 
@@ -785,6 +787,7 @@ _gst_compile_method (tree_node method,
   int primitiveIndex;
   int stack_depth;
   inc_ptr incPtr;
+  gst_compiled_method compiledMethod;
 
   dup_message_receiver = false;
   literal_vec_curr = literal_vec;
@@ -892,6 +895,9 @@ _gst_compile_method (tree_node method,
 					_gst_this_category,
 					method->location.file_offset,
 					method->v_method.endPos);
+
+      compiledMethod = (gst_compiled_method) OOP_TO_OBJ (methodOOP);
+      compiledMethod->header.isOldSyntax = method->v_method.isOldSyntax;
       INC_ADD_OOP (methodOOP);
 
       if (install)
@@ -1076,7 +1082,7 @@ compile_constant (tree_node constExpr)
      bytecode, or add it to the literals.  */
   if (index == -1)
     {
-      constantOOP = make_constant_oop (constExpr);
+      constantOOP = _gst_make_constant_oop (constExpr);
       if (IS_INT (constantOOP))
         {
           intVal = TO_INT (constantOOP);
@@ -1087,8 +1093,6 @@ compile_constant (tree_node constExpr)
 	      return;
 	    }
         }
-      else
-        MAKE_OOP_READONLY (constantOOP, true);
 
       index = add_literal (constantOOP);
     }
@@ -1942,7 +1946,14 @@ compile_assignments (tree_node varList)
 	_gst_compile_byte (STORE_RECEIVER_VARIABLE, variable.varIndex);
 
       else
-	_gst_compile_byte (STORE_LIT_VARIABLE, variable.varIndex);
+	{
+	  /* This can become a message send, which might not return the
+	     value.  Compile it in a way that can be easily peephole
+	     optimized. */
+	  _gst_compile_byte (STORE_LIT_VARIABLE, variable.varIndex);
+	  _gst_compile_byte (POP_STACK_TOP, 0);
+	  _gst_compile_byte (PUSH_LIT_VARIABLE, variable.varIndex);
+	}
     }
 }
 
@@ -2047,13 +2058,39 @@ equal_constant (OOP oop,
 	}
       break;
 
-    case CONST_OOP:
-      if (oop == constExpr->v_const.val.oopVal)
-	return (true);
+    case CONST_DEFERRED_BINDING:
+      if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_deferred_variable_binding_class)
+	{
+	  gst_deferred_variable_binding binding =
+	    (gst_deferred_variable_binding) OOP_TO_OBJ (oop);
+	  gst_object path = OOP_TO_OBJ (binding->path);
+	  int i, size = NUM_OOPS (path);
+	  OOP *pKey;
+	  tree_node varNode = constExpr->v_const.val.aVal;
+
+	  /* Use <= because we test the key first.  */
+	  for (i = 0, pKey = &binding->key; i <= size; pKey = &path->data[i++])
+	    {
+	      if (!varNode
+		  || *pKey != _gst_intern_string (varNode->v_list.name))
+		return (false);
+
+	      varNode = varNode->v_list.next;
+	      pKey = &path->data[i];
+	    }
+	}
       break;
 
     case CONST_BINDING:
-      if (oop == _gst_find_variable_binding (constExpr->v_const.val.aVal, false))
+      constExpr = _gst_find_variable_binding (constExpr->v_const.val.aVal);
+      if (!constExpr)
+	return (false);
+
+      assert (constExpr->v_const.constType != CONST_BINDING);
+      return equal_constant (oop, constExpr);
+
+    case CONST_OOP:
+      if (oop == constExpr->v_const.val.oopVal)
 	return (true);
       break;
 
@@ -2085,9 +2122,9 @@ equal_constant (OOP oop,
 }
 
 OOP
-make_constant_oop (tree_node constExpr)
+_gst_make_constant_oop (tree_node constExpr)
 {
-  tree_node arrayElt;
+  tree_node subexpr;
   int len, i;
   OOP resultOOP, elementOOP;
   inc_ptr incPtr;
@@ -2103,8 +2140,8 @@ make_constant_oop (tree_node constExpr)
 
   else if (constExpr->nodeType == TREE_ARRAY_ELT_LIST)
     {
-      for (len = 0, arrayElt = constExpr; arrayElt;
-	   len++, arrayElt = arrayElt->v_list.next);
+      for (len = 0, subexpr = constExpr; subexpr;
+	   len++, subexpr = subexpr->v_list.next);
 
       incPtr = INC_SAVE_POINTER ();
 
@@ -2114,15 +2151,14 @@ make_constant_oop (tree_node constExpr)
       instantiate_with (_gst_array_class, len, &resultOOP);
       INC_ADD_OOP (resultOOP);
 
-      for (i = 0, arrayElt = constExpr; i < len;
-	   i++, arrayElt = arrayElt->v_list.next)
+      for (i = 0, subexpr = constExpr; i < len;
+	   i++, subexpr = subexpr->v_list.next)
 	{
-	  elementOOP = make_constant_oop (arrayElt->v_list.value);
+	  elementOOP = _gst_make_constant_oop (subexpr->v_list.value);
 	  result = OOP_TO_OBJ (resultOOP);
 	  result->data[i] = elementOOP;
-	  if (IS_OOP (elementOOP))
-	    MAKE_OOP_READONLY (elementOOP, true);
 	}
+      MAKE_OOP_READONLY (resultOOP, true);
       INC_RESTORE_POINTER (incPtr);
       return (resultOOP);
     }
@@ -2145,47 +2181,80 @@ make_constant_oop (tree_node constExpr)
       return (floatq_new (constExpr->v_const.val.fVal));
 
     case CONST_STRING:
-      return (_gst_string_new (constExpr->v_const.val.sVal));
-
-    case CONST_OOP:
-      return (constExpr->v_const.val.oopVal);
+      resultOOP = _gst_string_new (constExpr->v_const.val.sVal);
+      MAKE_OOP_READONLY (resultOOP, true);
+      return (resultOOP);
 
     case CONST_BYTE_OBJECT:
       bo = constExpr->v_const.val.boVal;
       result = instantiate_with (bo->class, bo->size, &resultOOP);
       memcpy (result->data, bo->body, bo->size);
+      MAKE_OOP_READONLY (resultOOP, true);
       return (resultOOP);
 
+    case CONST_DEFERRED_BINDING:
+      {
+	gst_deferred_variable_binding dvb;
+	tree_node varNode = constExpr->v_const.val.aVal;
+
+        incPtr = INC_SAVE_POINTER ();
+        dvb = (gst_deferred_variable_binding)
+	  instantiate (_gst_deferred_variable_binding_class, &resultOOP);
+        INC_ADD_OOP (resultOOP);
+
+	dvb->key = _gst_intern_string (varNode->v_list.name);
+	dvb->class = _gst_this_class;
+	dvb->defaultDictionary = _gst_get_undeclared_dictionary ();
+	dvb->association = _gst_nil_oop;
+
+	varNode = varNode->v_list.next;
+	if (varNode)
+	  {
+	    int i, size = list_length (varNode);
+	    OOP arrayOOP;
+	    gst_object array =
+	      instantiate_with (_gst_array_class, size, &arrayOOP);
+
+	    dvb->path = arrayOOP;
+	    for (i = 0; i < size; i++, varNode = varNode->v_list.next)
+	      array->data[i] = _gst_intern_string (varNode->v_list.name);
+	  }
+
+        return (resultOOP);
+      }
+
     case CONST_BINDING:
-      resultOOP = _gst_find_variable_binding (constExpr->v_const.val.aVal,
-					      false);
-      if (IS_NIL (resultOOP))
+      subexpr = _gst_find_variable_binding (constExpr->v_const.val.aVal);
+      if (!subexpr)
 	{
 	  _gst_errorf_at (constExpr->location.first_line,
 			  "invalid variable binding");
           EXIT_COMPILATION ();
 	}
 
-      return (resultOOP);
+      assert (subexpr->v_const.constType != CONST_BINDING);
+      return _gst_make_constant_oop (subexpr);
+
+    case CONST_OOP:
+      return (constExpr->v_const.val.oopVal);
 
     case CONST_ARRAY:
-      for (len = 0, arrayElt = constExpr->v_const.val.aVal; arrayElt;
-	   len++, arrayElt = arrayElt->v_list.next);
+      for (len = 0, subexpr = constExpr->v_const.val.aVal; subexpr;
+	   len++, subexpr = subexpr->v_list.next);
 
       incPtr = INC_SAVE_POINTER ();
       result = instantiate_with (_gst_array_class, len, &resultOOP);
       INC_ADD_OOP (resultOOP);
 
-      for (i = 0, arrayElt = constExpr->v_const.val.aVal; i < len;
-	   i++, arrayElt = arrayElt->v_list.next)
+      for (i = 0, subexpr = constExpr->v_const.val.aVal; i < len;
+	   i++, subexpr = subexpr->v_list.next)
 	{
-	  elementOOP = make_constant_oop (arrayElt->v_list.value);
+	  elementOOP = _gst_make_constant_oop (subexpr->v_list.value);
 	  result = OOP_TO_OBJ (resultOOP);
 	  result->data[i] = elementOOP;
-	  if (IS_OOP (elementOOP))
-	    MAKE_OOP_READONLY (elementOOP, true);
 	}
 
+      MAKE_OOP_READONLY (resultOOP, true);
       INC_RESTORE_POINTER (incPtr);
       return (resultOOP);
     }
@@ -2294,27 +2363,28 @@ _gst_make_attribute (tree_node attribute_keywords)
 
   selectorOOP = _gst_compute_keyword_selector (attribute_keywords);
   numArgs = list_length (attribute_keywords);
-  argsArray = new_instance_with (_gst_array_class, numArgs, &argsArrayOOP);
+  argsArray = instantiate_with (_gst_array_class, numArgs, &argsArrayOOP);
   INC_ADD_OOP (argsArrayOOP);
 
   for (i = 0, keyword = attribute_keywords; keyword != NULL;
        i++, keyword = keyword->v_list.next)
     {
       tree_node value = keyword->v_list.value;
+      OOP result;
       if (value->nodeType != TREE_CONST_EXPR)
 	{
-          OOP result = _gst_execute_statements (NULL, value, UNDECLARED_NONE,
-					        true);
-          value = _gst_make_oop_constant (&value->location, result);
+          result = _gst_execute_statements (NULL, value, UNDECLARED_NONE, true);
           if (!result)
 	    {
 	      _gst_had_error = true;
 	      return _gst_nil_oop;
 	    }
 	}
+      else
+	result = _gst_make_constant_oop (value);
 
       argsArray = OOP_TO_OBJ (argsArrayOOP);
-      argsArray->data[i] = make_constant_oop (value);
+      argsArray->data[i] = result;
     }
 
   messageOOP = _gst_message_new_args (selectorOOP, argsArrayOOP);
@@ -2617,6 +2687,7 @@ _gst_make_new_method (int primitiveIndex,
   inc_ptr incPtr;
 
   maximumStackDepth += numArgs + numTemps;
+  memset (&header, 0, sizeof (method_header));
 
   incPtr = INC_SAVE_POINTER ();
   if (primitiveIndex)
@@ -2702,7 +2773,7 @@ _gst_make_new_method (int primitiveIndex,
     sourceCode = _gst_nil_oop;
   else
     {
-      sourceCode = file_segment_new (startPos, endPos);
+      sourceCode = _gst_get_source_string (startPos, endPos);
       INC_ADD_OOP (sourceCode);
     }
 
@@ -2795,6 +2866,8 @@ _gst_block_new (int numArgs,
   maximumStackDepth++;		/* just to be sure */
 
   numByteCodes = _gst_bytecode_length (bytecodes);
+
+  memset (&header, 0, sizeof (header));
   header.numArgs = numArgs;
   header.numTemps = numTemps;
   header.depth = maximumStackDepth;
@@ -2846,28 +2919,3 @@ method_info_new (OOP class,
   return (methodInfoOOP);
 }
 
-OOP
-file_segment_new (int64_t startPos, int64_t endPos)
-{
-  OOP fileName, fileSegmentOOP;
-  gst_file_segment fileSegment;
-  inc_ptr incPtr;
-
-  if (startPos == -1)
-    return _gst_get_cur_string ();
-
-  incPtr = INC_SAVE_POINTER ();
-  fileName = _gst_get_cur_file_name ();
-  INC_ADD_OOP (fileName);
-
-  fileSegment = (gst_file_segment) new_instance (_gst_file_segment_class,
-						 &fileSegmentOOP);
-
-  fileSegment->fileName = fileName;
-  fileSegment->startPos = from_c_int_64 (startPos);
-  fileSegment->length = from_c_int_64 (endPos - startPos);
-
-  assert (to_c_int_64 (fileSegment->length) >= 0);
-  INC_RESTORE_POINTER (incPtr);
-  return (fileSegmentOOP);
-}
