@@ -88,7 +88,7 @@
 #define MTH_DEPTH_BITS		6
 #define MTH_TEMPS_BITS		6
 #define MTH_ARGS_BITS		5
-#define MTH_PRIM_BITS		10
+#define MTH_PRIM_BITS		9
 #define MTH_FLAG_BITS		3
 
 #define MTH_NORMAL		0
@@ -108,8 +108,8 @@ typedef struct method_header
 #endif
   unsigned :1;			/* sign - must be 0 */
   unsigned headerFlag:MTH_FLAG_BITS;	/* prim _gst_self, etc.  */
-  unsigned primitiveIndex:MTH_PRIM_BITS;	/* index of primitve,
-						   or 0 */
+  unsigned isOldSyntax:1;
+  unsigned primitiveIndex:MTH_PRIM_BITS;	/* index of primitive, or 0 */
   unsigned numTemps:MTH_TEMPS_BITS;
   unsigned stack_depth:MTH_DEPTH_BITS;
   unsigned numArgs:MTH_ARGS_BITS;
@@ -121,6 +121,7 @@ typedef struct method_header
   unsigned numTemps:MTH_TEMPS_BITS;
   unsigned primitiveIndex:MTH_PRIM_BITS;	/* index of primitve,
 						   or 0 */
+  unsigned isOldSyntax:1;
   unsigned headerFlag:MTH_FLAG_BITS;	/* prim _gst_self, etc.  */
   unsigned :1;			/* sign - must be 0 */
 #if SIZEOF_OOP == 8
@@ -252,13 +253,6 @@ extern int _gst_declare_tracing
 extern mst_Boolean _gst_skip_compilation 
   ATTRIBUTE_HIDDEN;
 
-/* This is set to true by the parser or the compiler if an error
-   (respectively, a parse error or a semantic error) is found, and
-   avoids that _gst_execute_statements tries to execute the result of
-   the compilation.  */
-extern mst_Boolean _gst_had_error 
-  ATTRIBUTE_HIDDEN;
-
 /* This holds whether the compiler should make the compiled methods
    untrusted.  */
 extern mst_Boolean _gst_untrusted_methods 
@@ -269,6 +263,12 @@ extern mst_Boolean _gst_untrusted_methods
 extern OOP _gst_compute_keyword_selector (tree_node selectorExpr)
   ATTRIBUTE_HIDDEN;
 
+/* Given CONSTEXPR, a section of the syntax tree that represents a
+   Smalltalk constant, this routine creates and returns an OOP to be
+   stored as a method literal in the method that's currently being
+   compiled.  */ 
+extern OOP _gst_make_constant_oop (tree_node constExpr);
+
 /* Called to compile and execute an "immediate expression"; i.e. a Smalltalk
    statement that is not part of a method definition and where temporaries are
    declared automatically.  The parse trees are in TEMPS and STATEMENTS.
@@ -318,7 +318,7 @@ extern void _gst_reset_compilation_category ()
 
 /* This function will send a message to ObjectMemory (a system
    class) asking it to broadcast the event named HOOK.  */
-extern void _gst_invoke_hook (const char *hook) 
+extern void _gst_invoke_hook (enum gst_vm_hook hook) 
   ATTRIBUTE_HIDDEN;
 
 /* Prepares the compiler for execution, initializing some variables.  */
