@@ -130,9 +130,9 @@ typedef struct gst_identity_dictionary
 #define BEHAVIOR_HEADER \
   OBJ_HEADER; \
   OOP		superclass; \
-  OOP		subClasses; \
   OOP		methodDictionary; \
   intptr_t	instanceSpec; \
+  OOP		subClasses; \
   OOP		instanceVariables
 
 typedef struct gst_behavior
@@ -158,12 +158,51 @@ typedef struct gst_behavior
 #define COBJECT_INT_TYPE		FROM_INT(10)
 #define COBJECT_UNSIGNED_INT_TYPE	FROM_INT(11)
 
+/*    3                   2                   1
+    1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+   |    # fixed fields             |      unused       |I| kind  |1|
+   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+   I'm moving it to bits 13-30 (it used to stay at bits 5-30),
+   allocating space for more flags in case they're needed.
+   If you change ISP_NUMFIXEDFIELDS you should modify Behavior.st too.
+   Remember to shift by ISP_NUMFIXEDFIELDS-1 there, since Smalltalk does
+   not see GST_ISP_INTMARK!!  */
+
+enum {
+  ISP_NUMFIXEDFIELDS = 13,
+
+  /* Set if the instances of the class have indexed instance
+     variables.  */
+  ISP_ISINDEXABLE = 32,
+
+  /* These represent the shape of the indexed instance variables of
+     the instances of the class.  */
+  ISP_INDEXEDVARS = 62,
+  ISP_SHAPE = 30,
+
+  /* Set to 1 to mark a SmallInteger.  */
+  GST_ISP_INTMARK = 1
+};
+
 typedef struct gst_class_description
 {
   CLASS_DESCRIPTION_HEADER;
 }
  *gst_class_description;
 
+typedef struct gst_deferred_variable_binding
+{
+  OBJ_HEADER;
+  OOP key;
+  OOP class;
+  OOP defaultDictionary;
+  OOP association;
+  OOP path;
+}
+ *gst_deferred_variable_binding;
+
 typedef struct gst_association
 {
   OBJ_HEADER;
@@ -280,6 +319,7 @@ typedef struct gst_cobject
 {
   OBJ_HEADER;
   OOP type;
+  OOP storage;
 }
  *gst_cobject;
 
@@ -320,6 +360,7 @@ extern OOP _gst_compiled_method_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_context_part_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_continuation_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_date_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_deferred_variable_binding_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_delay_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_dictionary_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_directed_message_class ATTRIBUTE_HIDDEN;
@@ -504,21 +545,14 @@ extern OOP _gst_shared_pool_dictionary (OOP class_oop)
   ATTRIBUTE_PURE 
   ATTRIBUTE_HIDDEN;
 
-/* Creates a new CObject pointing to cObjPtr, extracting the name of
-   the class to be instantiated from the CType, TYPEOOP.  */
-extern OOP _gst_c_object_new_typed (PTR cObjPtr,
-				    OOP typeOOP) 
-  ATTRIBUTE_HIDDEN;
-
-/* Allocates a new CObject by malloc-ing SIZE bytes; CLASS_OOP is the
-   class to be instantiated.  */
-extern OOP _gst_alloc_cobject (OOP class_oop,
-			       size_t size) 
-  ATTRIBUTE_HIDDEN;
-
-/* Creates a new CType that when passed to _gst_c_object_new_typed
-   creates an instance of COBJECTSUBCLASSOOP.  */
-extern OOP _gst_c_type_new (OOP cObjectSubclassOOP) 
+/* Creates a new CObject pointing to cObjOfs bytes in BASEOOP (or
+   at the absolute address cObjOfs if BASEOOP is NULL), extracting the class
+   to be instantiated from the CType, TYPEOOP, or using the provided
+   class if TYPEOOP is nil.  */
+extern OOP _gst_c_object_new_base (OOP baseOOP,
+			           uintptr_t cObjOfs,
+			           OOP typeOOP,
+			           OOP defaultClassOOP) 
   ATTRIBUTE_HIDDEN;
 
 /* Creates a new String with LEN indexed instance variables.  */
