@@ -6,7 +6,7 @@
 
 /***********************************************************************
  *
- * Copyright 2005, 2006 Free Software Foundation, Inc.
+ * Copyright 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * Written by Paolo Bonzini.
  *
  * This file is part of GNU Smalltalk.
@@ -97,7 +97,8 @@ static int filprintf (Filament *fil,
 /* Grammar productions.  */
 
 static void parse_chunks (gst_parser *p);
-static void parse_doit (gst_parser *p);
+static void parse_doit (gst_parser *p,
+			mst_Boolean fail_at_eof);
 static mst_Boolean parse_scoped_definition (gst_parser *p, 
 					    tree_node first_stmt);
 
@@ -107,13 +108,15 @@ static void parse_eval_definition (gst_parser *p);
 static mst_Boolean parse_namespace_definition (gst_parser *p,
 					       tree_node first_stmt);
 static mst_Boolean parse_class_definition (gst_parser *p,
-					   OOP classOOP);
+					   OOP classOOP,	
+					   mst_Boolean extend);
 static OOP parse_namespace (tree_node name);
 static OOP parse_class (tree_node list);
 static void parse_scoped_method (gst_parser *p,
 				 OOP classOOP);
 static void parse_instance_variables (gst_parser *p,
-				      OOP classOOP);
+				      OOP classOOP,	
+				      mst_Boolean extend);
 
 static void parse_method_list (gst_parser *p);
 static void parse_method (gst_parser *p,
@@ -121,13 +124,15 @@ static void parse_method (gst_parser *p,
 static tree_node parse_message_pattern (gst_parser *p);
 static tree_node parse_keyword_variable_list (gst_parser *p);
 static tree_node parse_variable (gst_parser *p);
-static tree_node parse_attributes (gst_parser *p);
+static tree_node parse_attributes (gst_parser *p,
+				  tree_node prev_attrs);
 static tree_node parse_attribute (gst_parser *p);
 static tree_node parse_temporaries (gst_parser *p,
 				    mst_Boolean implied_pipe);
 static tree_node parse_statements (gst_parser *p,
 				   tree_node first_stmt,
 				   mst_Boolean accept_caret);
+static tree_node parse_required_expression (gst_parser *p);
 static tree_node parse_expression (gst_parser *p,
 				   enum expr_kinds kind);
 static tree_node parse_primary (gst_parser *p);
@@ -309,7 +314,7 @@ _gst_parse_method ()
   p.state = PARSE_METHOD;
   lex_init (&p);
   if (setjmp (p.recover) == 0)
-    parse_method (&p, EOF);
+    parse_method (&p, ']');
   else
     _gst_had_error = false;
 
@@ -344,16 +349,22 @@ parse_chunks (gst_parser *p)
   else
     {
       OOP oldTemporaries = _gst_push_temporaries_dictionary ();
-      while (token (p, 0) != EOF && token (p, 0) != ']' && token (p, 0) != '!')
+      jmp_buf old_recover;
+      memcpy (old_recover, p->recover, sizeof (p->recover));
+      setjmp (p->recover);
+      while (token (p, 0) != EOF && token (p, 0) != '!')
         {
           /* Pick the production here, so that subsequent 
 	     methods are compiled when we come back from an error above.  */
           if (p->state == PARSE_METHOD_LIST)
 	    parse_method_list (p);
           else
-	    parse_doit (p);
+	    parse_doit (p, false);
         }
+
+      lex_skip_if (p, '!', false);
       _gst_pop_temporaries_dictionary (oldTemporaries);
+      memcpy (p->recover, old_recover, sizeof (p->recover));
     }
 }
 
@@ -386,7 +397,6 @@ expected (gst_parser *p, int token, ...)
       token = va_arg (ap, int);
     }
 
-#define TOKEN_SEP
 #define TOKEN_DEF(name, val, str, subsume)				\
   if ((named_tokens & (1 << (val - FIRST_TOKEN))) != 0			\
       && (subsume == -1							\
@@ -398,7 +408,6 @@ expected (gst_parser *p, int token, ...)
 
   TOKEN_DEFS
 #undef TOKEN_DEF
-#undef TOKEN_SEP
 
     msg = fildelete (out_fil);
   _gst_errorf ("%s", msg);
@@ -413,19 +422,18 @@ static void
 recover_error (gst_parser *p)
 {
   if (p->state != PARSE_METHOD)
-    for (;;)
-      {
-	/* Find the final bang.  */
-	if (token (p, 0) == EOF)
-	  break;
-	if (token (p, 0) == '!')
-	  {
-	    _gst_free_tree ();
-	    lex (p);
-	    break;
-	  }
+    {
+      _gst_error_recovery = true;
+
+      /* Find the final bang or, if in the REPL, a newline.  */
+      while (token (p, 0) != EOF
+	     && token (p, 0) != '!'
+	     && token (p, 0) != ERROR_RECOVERY)
 	lex (p);
-      }
+
+      _gst_error_recovery = false;
+      lex_skip_if (p, ERROR_RECOVERY, false);
+    }
 
   longjmp (p->recover, 1);
 }
@@ -434,7 +442,7 @@ recover_error (gst_parser *p)
    | empty */
 
 static void
-parse_doit (gst_parser *p)
+parse_doit (gst_parser *p, mst_Boolean fail_at_eof)
 {
   tree_node statement = NULL;
   mst_Boolean caret;
@@ -442,11 +450,11 @@ parse_doit (gst_parser *p)
   if (token (p, 0) == '|')
     parse_temporaries (p, false);
 
-  if (token (p, 0) == EOF)
+  if (token (p, 0) == EOF && !fail_at_eof)
     return;
 
   caret = lex_skip_if (p, '^', false);
-  statement = parse_expression (p, EXPR_ANY);
+  statement = parse_required_expression (p);
   if (!caret && lex_skip_if (p, '[', false))
     {
       if (parse_scoped_definition (p, statement))
@@ -467,7 +475,6 @@ parse_doit (gst_parser *p)
 
   /* Do not lex until after _gst_free_tree, or we lose a token! */
   lex_skip_if (p, '.', false);
-  lex_skip_if (p, '!', false);
 }
 
 
@@ -511,41 +518,68 @@ parse_scoped_definition (gst_parser *p, tree_node first_stmt)
 	  if (IS_NIL (classOOP))
 	    _gst_had_error = true;
 	  else
-	    return parse_class_definition (p, classOOP);  
+	    return parse_class_definition (p, classOOP, false);  
 	}
     }
 
   else if (first_stmt->nodeType == TREE_UNARY_EXPR
 	   && first_stmt->v_expr.selector == _gst_intern_string ("extend"))
     {
-      
-      if (receiver->nodeType == TREE_VARIABLE_NODE)
-	classOOP = parse_class (receiver);
+      OOP namespace_old = _gst_current_namespace;
+      OOP classOrMetaclassOOP = NULL;
+      mst_Boolean ret_value;
 
+      _gst_register_oop (namespace_old);
+      if (receiver->nodeType == TREE_VARIABLE_NODE)
+ 	{
+ 	  classOOP = parse_class (receiver);
+	  classOrMetaclassOOP = classOOP;
+ 	}
       else if (receiver->nodeType == TREE_UNARY_EXPR
 	       && receiver->v_expr.selector == _gst_intern_string ("class"))
-	{
-	  classOOP = parse_class (receiver->v_expr.receiver);
-	  classOOP = OOP_CLASS (classOOP);
-	}
+ 	{
+ 	  classOOP = parse_class (receiver->v_expr.receiver);
+	  classOrMetaclassOOP = OOP_CLASS (classOOP);
+ 	}	   
+      if (classOrMetaclassOOP != NULL) 
+ 	{ 
+ 	  OOP namespace_new = ((gst_class) OOP_TO_OBJ (classOOP))->environment; 
+	  
+	  /* When creating the image, current namespace is not available. */
+	  if (namespace_new != namespace_old)
+	    _gst_msg_sendf (NULL, "%v %o current: %o",
+			    _gst_namespace_class, namespace_new);
+	  
+ 	  ret_value = parse_class_definition (p, classOrMetaclassOOP, true);
 
-      if (classOOP != NULL)
-	return parse_class_definition (p, classOOP);
+	  if (namespace_new != namespace_old)
+	    _gst_msg_sendf (NULL, "%v %o current: %o",
+			    _gst_namespace_class, namespace_old);
+	  
+ 	  _gst_unregister_oop (namespace_old);
+ 	  return ret_value;
+ 	}
     }
 
-  _gst_errorf ("expected Eval, Namespace or class definition"); 
+  _gst_errorf_at (first_stmt->location.first_line, 
+		  "expected Eval, Namespace or class definition"); 
   return false;
 }
 
 static void
 parse_eval_definition (gst_parser *p)
 {
-  tree_node tmps = NULL, stmts;
-  tree_node first_stmt = NULL;
+  tree_node tmps = NULL, stmts = NULL;
   OOP oldDictionary = _gst_push_temporaries_dictionary ();
+  jmp_buf old_recover;
+
+  memcpy (old_recover, p->recover, sizeof (p->recover));
+  if (setjmp (p->recover) == 0)
+    {
+      tmps = parse_temporaries (p, false);
+      stmts = parse_statements (p, NULL, true);
+    }
 
-  tmps = parse_temporaries (p, false);
-  stmts = parse_statements (p, first_stmt, true);
   if (stmts && !_gst_had_error)
     {
       if (_gst_regression_testing)
@@ -556,11 +590,14 @@ parse_eval_definition (gst_parser *p)
 
       if (_gst_regression_testing && !_gst_had_error)
         printf ("returned value is %O\n", _gst_last_returned_value);
+      _gst_had_error = false;
     }
 
-  _gst_pop_temporaries_dictionary (oldDictionary);
   _gst_free_tree ();
-  _gst_had_error = false;
+  _gst_pop_temporaries_dictionary (oldDictionary);
+  memcpy (p->recover, old_recover, sizeof (p->recover));
+  if (_gst_had_error)
+    longjmp (p->recover, 1);
 }
 
 static mst_Boolean
@@ -577,8 +614,9 @@ parse_namespace_definition (gst_parser *p, tree_node first_stmt)
       _gst_msg_sendf (NULL, "%v %o current: %o", 
 		      _gst_namespace_class, new_namespace);
       
-      parse_eval_definition (p);	
-      
+      while (token (p, 0) != ']' && token (p, 0) != EOF && token (p, 0) != '!')
+        parse_doit (p, true);	
+
       _gst_msg_sendf (NULL, "%v %o current: %o",
 		      _gst_namespace_class, old_namespace);
       
@@ -590,10 +628,11 @@ parse_namespace_definition (gst_parser *p, tree_node first_stmt)
 }
 
 static mst_Boolean
-parse_class_definition (gst_parser *p, OOP classOOP)
+parse_class_definition (gst_parser *p, OOP classOOP, mst_Boolean extend)
 {
   int t1, t2, t3;
-  
+  mst_Boolean add_inst_vars = extend;
+
   for (;;)
     {
       if (_gst_had_error)
@@ -624,7 +663,7 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 	  _gst_set_compilation_class (classOOP);
 	  parse_method (p, ']');
 	  _gst_reset_compilation_category ();
-	  break;
+	  continue;
 	  
 	case '<':
 	  if (t2 == IDENTIFIER) 
@@ -636,6 +675,7 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 	      _gst_set_compilation_class (classOOP);
 	      parse_method (p, ']');
 	      _gst_reset_compilation_category ();
+	      continue;
 	    }
 	  else if (t2 == KEYWORD) 
 	    {
@@ -648,7 +688,10 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 	      lex_skip_mandatory (p, '<');
 	      keyword = parse_keyword_expression (p, NULL, EXPR_KEYWORD);
 	      if (keyword->v_expr.expression->v_list.next != NULL)
-		_gst_errorf ("expected one keyword only");
+		{
+		  _gst_errorf ("expected one keyword only");
+		  _gst_had_error = true;
+		}
 	      else
 		{
 		  value = keyword->v_expr.expression->v_list.value;
@@ -665,11 +708,10 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 		}
 
 	      lex_skip_mandatory (p, '>');
+	      continue;
 	    }
-	  else
-	    _gst_errorf ("invalid class body element");
 	  break;
-	  
+   
 	case IDENTIFIER:
 	  if (t2 == ASSIGNMENT)
 	    {
@@ -679,25 +721,38 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 
  	      OOP name, class_var_dict, result;
 	      tree_node stmt;
+	      OOP the_class = classOOP;
+	      if (IS_A_METACLASS (classOOP))
+		the_class = METACLASS_INSTANCE (classOOP);
 
 	      name = _gst_intern_string (val (p, 0)->sval);
 
 	      lex_skip_mandatory (p, IDENTIFIER);
 	      lex_skip_mandatory (p, ASSIGNMENT);
 
-	      class_var_dict = _gst_class_variable_dictionary (classOOP);
+	      class_var_dict = _gst_class_variable_dictionary (the_class);
+	      if (IS_NIL (class_var_dict))
+		{
+		  gst_class class;
+		  class_var_dict = _gst_binding_dictionary_new (8, the_class);
+		  class = (gst_class) OOP_TO_OBJ (the_class);
+		  class->classVariables = class_var_dict;
+		}
 	      
-	      stmt = parse_expression (p, EXPR_ANY);
-	      stmt = _gst_make_statement_list (&stmt->location, stmt);
-	      result = _gst_execute_statements (NULL, stmt, UNDECLARED_NONE,
-						true);
-
-	      assert (result || _gst_had_error);
+	      stmt = parse_required_expression (p);
 	      if (!_gst_had_error)
-		DICTIONARY_AT_PUT (class_var_dict, name, result);
+		{
+	          stmt = _gst_make_statement_list (&stmt->location, stmt);
+	          result = _gst_execute_statements (NULL, stmt, UNDECLARED_NONE,
+						    true);
+
+	          assert (result);
+		  DICTIONARY_AT_PUT (class_var_dict, name, result);
+		}
 
 	      if (token (p, 0) != ']')
 		lex_skip_mandatory(p, '.');	      
+	      continue;
 	    }
 	  else if (t2 == BINOP)
 	    {
@@ -705,6 +760,7 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 	      printf ("parse method\n");
 #endif
 	      parse_scoped_method (p, classOOP);
+	      continue;
 	    }
 	  else if (t2 == '[')
 	    {
@@ -715,6 +771,7 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 	      _gst_set_compilation_class (classOOP);
 	      parse_method (p, ']');
 	      _gst_reset_compilation_category ();
+	      continue;
 	    }
 	  else if (t2 == SCOPE_SEPARATOR)
 	    {
@@ -723,6 +780,7 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 #endif
 
 	      parse_scoped_method (p, classOOP);
+	      continue;
 	    }
 	  else if (t2 == IDENTIFIER)
 	    {
@@ -732,28 +790,34 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 		  printf ("parse class method\n");
 #endif
 		  parse_scoped_method (p, classOOP);
+		  continue;
 		}
-	      else if (t3 == '['
-		       && strcmp (val (p, 0)->sval, "Class") == 0
-		       && strcmp (val (p, 1)->sval, "protocol") == 0)
+	      else if (t3 == '[' && strcmp (val (p, 1)->sval, "class") == 0)
 		{
 #if 0
 		  printf ("parse class protocol\n");
 #endif
 		  if (_gst_object_is_kind_of (classOOP, _gst_metaclass_class))
-		    _gst_errorf ("already on class side, Class protocol invalid");
+		    {
+		      _gst_errorf ("already on class side");
+		      _gst_had_error = true;
+		      continue;
+		    }
+		  else if (((gst_class) OOP_TO_OBJ (classOOP))->name
+			   != _gst_intern_string (val (p, 0)->sval))
+		    {
+		      _gst_errorf ("`%s class' invalid within %O",
+				   val (p, 0)->sval, classOOP);
+		      _gst_had_error = true;
+		      continue;
+		    }
 		  else
 		    {
 		      lex_consume (p, 3);
-		      parse_class_definition (p, OOP_CLASS (classOOP));
+		      parse_class_definition (p, OOP_CLASS (classOOP), extend);
 		      lex_skip_mandatory (p, ']');
 		    }
-		}
-	      else 
-		{
-		  _gst_errorf("invalid class body element");
-		  lex_consume (p, p->la_size);
-		  return false;
+		  continue;
 		}
 	    }
 	  break;
@@ -764,7 +828,8 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 #if 0
 	      printf ("parse instance variables - ignore\n");
 #endif
-	      lex_consume (p, 2);	      
+	      lex_consume (p, 2);
+	      continue;
 	    }
 	  else if (t2 == IDENTIFIER) 
 	    {
@@ -773,7 +838,9 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 #if 0
 		  printf ("parse instance variables\n");
 #endif
-		  parse_instance_variables (p, classOOP);
+		  parse_instance_variables (p, classOOP, add_inst_vars);
+		  add_inst_vars = true;
+		  continue;
 		}
 	      else if (t3 == '[')
 		{
@@ -783,13 +850,20 @@ parse_class_definition (gst_parser *p, OOP classOOP)
 		  _gst_set_compilation_class (classOOP);
 		  parse_method (p, ']');
 		  _gst_reset_compilation_category ();
+		  continue;
 		}
-	    }	  
+	    }   
+	  break;
+   
+	default:
 	  break;
 	}
+
+      _gst_errorf ("invalid class body element");
+      _gst_had_error = true;
     }
-  
-  return true;
+
+  return !_gst_had_error;
 }
 
 static void 
@@ -819,19 +893,30 @@ parse_scoped_method (gst_parser *p, OOP classOOP)
   else
     _gst_errorf ("expected `>>'");
   
-  if (_gst_class_is_kind_of (classOOP, class))
+  if (!class)
+    {
+      _gst_skip_compilation = true;
+      class = classOOP;
+    }
+
+  else if (!_gst_class_is_kind_of (classOOP, class))
+    {
+      _gst_skip_compilation = true;
+      _gst_errorf ("%#O is not %#O or one of its superclasses",
+		   ((gst_class) OOP_TO_OBJ (class))->name,
+		   ((gst_class) OOP_TO_OBJ (classOOP))->name);
+    }
+
+  else
     {
       if (class_method) 
-	  class = OOP_CLASS (class);
-
-      _gst_set_compilation_class (class);
-      parse_method (p, ']');
-      _gst_reset_compilation_category ();
+	class = OOP_CLASS (class);
     }
-  else
-    _gst_errorf ("%O is not %O or one of its superclasses",
-		 ((gst_class) OOP_TO_OBJ (class))->name,
-		 ((gst_class) OOP_TO_OBJ (classOOP))->name);
+
+  _gst_set_compilation_class (class);
+  parse_method (p, ']');
+  _gst_reset_compilation_category ();
+  _gst_skip_compilation = false;
 }
 
 static OOP
@@ -839,25 +924,29 @@ parse_class (tree_node list)
 {
   const char* name;
   OOP currentOOP = _gst_current_namespace;
-	
+  tree_node next; 
+
+  if (strcmp (list->v_list.name, "nil") == 0)
+      return _gst_nil_oop;
+  
   do
     {
       name = list->v_list.name;
-      currentOOP = dictionary_at (currentOOP, _gst_intern_string (name));
+      currentOOP = _gst_namespace_at (currentOOP, _gst_intern_string (name));
 	
       if (currentOOP == _gst_nil_oop)
 	{
-	  _gst_errorf ("key %s not found", name);
+	  _gst_errorf_at (list->location.first_line, "key %s not found", name);
 	  return NULL;
 	}
 
-      list = list->v_list.next;
-
-      if (list == NULL)
+      next = list->v_list.next;
+      if (next == NULL)
 	{
 	  if (!_gst_object_is_kind_of (currentOOP, _gst_class_class)) 
 	    {
-	      _gst_errorf ("expected class named %s, found %O", 
+	      _gst_errorf_at (list->location.first_line,
+			      "expected class named %s, found %O", 
 			   name, OOP_INT_CLASS (currentOOP));
 	      return NULL;
 	    }
@@ -866,11 +955,13 @@ parse_class (tree_node list)
 	{
 	  if (!_gst_object_is_kind_of (currentOOP, _gst_dictionary_class))
 	    {
-	      _gst_errorf ("expected namespace named %s, found %O", 
-			   name, OOP_INT_CLASS (currentOOP));
+	      _gst_errorf_at  (list->location.first_line,
+			       "expected namespace named %s, found %O", 
+			       name, OOP_INT_CLASS (currentOOP));
 	      return NULL;
 	    }
 	}
+      list = next;
     }
   while (list != NULL);
 		
@@ -887,18 +978,20 @@ parse_namespace (tree_node list)
   while (list->v_list.next != NULL)
     {
       name = _gst_intern_string (list->v_list.name);	
-      current_namespace = dictionary_at (current_namespace, name);
+      current_namespace = _gst_namespace_at (current_namespace, name);
 		
       if (current_namespace == _gst_nil_oop)
 	{
-	  _gst_errorf ("key %s not found", list->v_list.name);
+	  _gst_errorf_at (list->location.first_line, 
+			  "key %s not found", list->v_list.name);
 	  return NULL;
 	}
       
       if (!_gst_object_is_kind_of (current_namespace, _gst_dictionary_class))
 	{
-	  _gst_errorf ("expected namespace named %s, found %O", 
-		       list->v_list.name, OOP_INT_CLASS (current_namespace));
+	  _gst_errorf_at (list->location.first_line, 
+			  "expected namespace named %s, found %O", 
+			  list->v_list.name, OOP_INT_CLASS (current_namespace));
 	  return NULL;
 	}
 
@@ -917,8 +1010,9 @@ parse_namespace (tree_node list)
     current_namespace = new_namespace;
 
   else
-    _gst_errorf ("expected namespace named %s, found %o", namespc,
-		 OOP_INT_CLASS (new_namespace));
+    _gst_errorf_at (list->location.first_line, 
+		    "expected namespace named %s, found %O", namespc,
+		    OOP_INT_CLASS (new_namespace));
 
   return current_namespace;
 }
@@ -927,11 +1021,31 @@ parse_namespace (tree_node list)
    | empty */
 
 static void
-parse_instance_variables (gst_parser *p, OOP classOOP)
+parse_instance_variables (gst_parser *p, OOP classOOP, mst_Boolean extend)
 {
   char *vars;
   Filament *fil = filnew (NULL, 0);
   
+  if (extend)
+    {
+      gst_behavior class = (gst_behavior) OOP_TO_OBJ (classOOP);
+      OOP *instVars = OOP_TO_OBJ (class->instanceVariables)->data;
+      int n = CLASS_FIXED_FIELDS (classOOP);
+      OOP superclassOOP = SUPERCLASS (classOOP);
+      if (!IS_NIL (superclassOOP))
+	{
+	  int superclassVars = CLASS_FIXED_FIELDS (superclassOOP);
+	  instVars += superclassVars;
+	  n -= superclassVars;
+	}
+      for (; n--; instVars++)
+	{
+	  char *s = _gst_to_cstring (*instVars);
+          filprintf (fil, "%s ", s);
+	  xfree (s);
+	}
+    }
+
   lex_skip_mandatory (p, '|');
   while (!lex_skip_if (p, '|', true))
     {
@@ -951,7 +1065,6 @@ parse_method_list (gst_parser *p)
   while (token (p, 0) != '!')
     parse_method (p, '!');
 
-  lex (p);
   _gst_skip_compilation = false;
   _gst_reset_compilation_category ();
   p->state = PARSE_DOIT;
@@ -963,7 +1076,7 @@ parse_method_list (gst_parser *p)
 static void
 parse_method (gst_parser *p, int at_end)
 {
-  tree_node pat, temps, attrs, stmts;
+  tree_node pat, temps, stmts, attrs = NULL;
   YYLTYPE current_pos;
   tree_node method;
 
@@ -972,16 +1085,27 @@ parse_method (gst_parser *p, int at_end)
   if (at_end == ']')
     lex_skip_mandatory (p, '[');
 
+  if (token (p, 0) == '<')
+    attrs = parse_attributes (p, NULL);
+
   temps = parse_temporaries (p, false);
-  attrs = parse_attributes (p);
+
+  if (token (p, 0) == '<')
+    attrs = parse_attributes (p, attrs);
+
   stmts = parse_statements (p, NULL, true);
 
   /* Don't lex until _gst_free_tree, or we lose a token.  */
   lex_must_be (p, at_end);
 
+  /* Still, include the ']' in the method source code.  */
   current_pos = _gst_get_location ();
+  if (at_end == ']')
+    current_pos.file_offset++;
+
   method = _gst_make_method (&pat->location, &current_pos,
-			     pat, temps, attrs, stmts);
+			     pat, temps, attrs, stmts,
+			     at_end != ']');
 
   if (!_gst_had_error && !_gst_skip_compilation)
     {
@@ -1083,18 +1207,16 @@ parse_variable (gst_parser *p)
    | KEYWORD binary_expr */
 
 static tree_node
-parse_attributes (gst_parser *p)
+parse_attributes (gst_parser *p, tree_node prev_attrs)
 {
-  tree_node attrs = NULL;
-
   while (token (p, 0) == '<')
     {
       tree_node attr = parse_attribute (p);
       if (attr)
-	attrs = _gst_add_node (attrs, attr);
+	prev_attrs = _gst_add_node (prev_attrs, attr);
     }
 
-  return attrs;
+  return prev_attrs;
 }
 
 static tree_node
@@ -1162,18 +1284,14 @@ parse_statements (gst_parser *p, tree_node first_stmt, mst_Boolean accept_caret)
   do
     {
       caret = accept_caret && lex_skip_if (p, '^', false);
-      stmt = parse_expression (p, EXPR_ANY);
       if (caret)
 	{
-	  if (stmt == NULL)
-	    {
-	      _gst_errorf ("expected statement after '^'");
-	      recover_error (p);
-	    }
+          stmt = parse_required_expression (p);
 	  stmt = _gst_make_return (&stmt->location, stmt);
 	}
       else
 	{
+          stmt = parse_expression (p, EXPR_ANY);
 	  if (stmt == NULL)
 	    break;
 	}
@@ -1216,16 +1334,35 @@ parse_expression (gst_parser *p, enum expr_kinds kind)
 							  node));
     }
 
-  if (!node)
-    return node;
+  if (!node && assigns)
+    {
+      _gst_errorf ("expected expression");
+      recover_error (p);
+    }
+
+  if (node)
+    {
+      node = parse_message_expression (p, node, kind & ~EXPR_ASSIGNMENT);
+      assert (node);
+    }
 
-  node = parse_message_expression (p, node, kind & ~EXPR_ASSIGNMENT);
   if (assigns)
-    return _gst_make_assign (&assigns->location, assigns, node);
-  else
-    return node;
+    node = _gst_make_assign (&assigns->location, assigns, node);
+
+  return node;
 }
 
+static tree_node
+parse_required_expression (gst_parser *p)
+{
+  tree_node stmt = parse_expression (p, EXPR_ANY);
+  if (!stmt)
+    {
+      _gst_errorf ("expected expression");
+      recover_error (p);
+    }
+  return stmt;
+}
 
 /* primary: variable_primary
    | '(' expression ')'
@@ -1267,7 +1404,7 @@ parse_primary (gst_parser *p)
 
     case '(':
       lex (p);
-      node = parse_expression (p, EXPR_ANY);
+      node = parse_required_expression (p);
       lex_skip_mandatory (p, ')');
       break;
 
@@ -1564,9 +1701,10 @@ parse_array_constructor (gst_parser *p)
 }
 
 
-/* block: block_vars '||' temps_no_pipe statements
-   | block_vars '|' temporaries statements
-   | temporaries statements */
+/* block: '[' block_vars '||' temps_no_pipe statements ']'
+   | '[' block_vars '|' temporaries statements ']'
+   | '[' block_vars ']'
+   | '[' temporaries statements ']' */
 
 static tree_node
 parse_block (gst_parser *p)
@@ -1581,7 +1719,9 @@ parse_block (gst_parser *p)
   if (token (p, 0) == ':')
     {
       vars = parse_block_variables (p);
-      if (lex_skip_if (p, '|', true))
+      if (token (p, 0) == ']')
+	implied_pipe = false;
+      else if (lex_skip_if (p, '|', true))
 	implied_pipe = false;
       else if (token (p, 0) == BINOP
 	       && val(p, 0)->sval[0] == '|' && val(p, 0)->sval[1] == '|')
@@ -1590,7 +1730,7 @@ parse_block (gst_parser *p)
 	  lex (p);
 	}
       else
-	expected (p, ':', '|', -1);
+	expected (p, ':', '|', ']', -1);
     }
   else
     {
