@@ -176,4 +176,77 @@ struct object_s
 #define IS_OOP(oop) \
   (! IS_INT(oop) )
 
+/* Keep these in sync with _gst_sizes, in dict.c.
+   FIXME: these should be exported in a pool dictionary.  */
+enum gst_indexed_kind {
+  GST_ISP_FIXED = 0,
+  GST_ISP_SCHAR = 32,
+  GST_ISP_UCHAR = 34,
+  GST_ISP_CHARACTER = 36,
+  GST_ISP_SHORT = 38,
+  GST_ISP_USHORT = 40,
+  GST_ISP_INT = 42,
+  GST_ISP_UINT = 44,
+  GST_ISP_FLOAT = 46,
+  GST_ISP_INT64 = 48,
+  GST_ISP_UINT64 = 50,
+  GST_ISP_DOUBLE = 52,
+  GST_ISP_UTF32 = 54,
+  GST_ISP_LAST_SCALAR = 54,
+  GST_ISP_POINTER = 62,
+
+#if SIZEOF_OOP == 8
+  GST_ISP_LONG = GST_ISP_INT64,
+  GST_ISP_ULONG = GST_ISP_UINT64,
+  GST_ISP_LAST_UNALIGNED = GST_ISP_FLOAT,
+#else
+  GST_ISP_LONG = GST_ISP_INT,
+  GST_ISP_ULONG = GST_ISP_UINT,
+  GST_ISP_LAST_UNALIGNED = GST_ISP_USHORT,
+#endif
+};
+
+
+/* enum types used by the public APIs.  */
+enum gst_file_dir {
+  GST_DIR_ABS,
+  GST_DIR_KERNEL_SYSTEM,
+  GST_DIR_KERNEL,
+  GST_DIR_BASE
+};
+
+enum gst_var_index {
+  GST_DECLARE_TRACING,
+  GST_EXECUTION_TRACING,
+  GST_EXECUTION_TRACING_VERBOSE,
+  GST_GC_MESSAGE,
+  GST_VERBOSITY,
+  GST_MAKE_CORE_FILE,
+  GST_REGRESSION_TESTING
+};
+
+enum gst_init_flags {
+  GST_REBUILD_IMAGE = 1,
+  GST_MAYBE_REBUILD_IMAGE = 2,
+  GST_IGNORE_USER_FILES = 4,
+  GST_IGNORE_BAD_IMAGE_PATH = 8,
+  GST_IGNORE_BAD_KERNEL_PATH = 16,
+  GST_NO_TTY = 32,
+};
+
+enum gst_vm_hook {
+  GST_BEFORE_EVAL,
+  GST_AFTER_EVAL,
+  GST_RETURN_FROM_SNAPSHOT,
+  GST_ABOUT_TO_QUIT,
+  GST_ABOUT_TO_SNAPSHOT,
+  GST_FINISHED_SNAPSHOT
+};
+
+#define INDEXED_WORD(obj, n)   ( ((long *) ((obj) + 1))		    [(n)-1] )
+#define INDEXED_BYTE(obj, n)   ( ((char *) ((obj) + 1))		    [(n)-1] )
+#define INDEXED_OOP(obj, n)    ( ((OOP  *) ((obj) + 1))		    [(n)-1] )
+#define ARRAY_OOP_AT(obj, n)   ( ((OOP  *) ((gst_object) obj)->data) [(n)-1] )
+#define STRING_OOP_AT(obj, n)  ( ((char *) ((gst_object) obj)->data) [(n)-1] )
+
 #endif /* GST_GST_H */
