@@ -643,16 +643,15 @@ extern OOP _gst_nil_oop
 #include "rbtrees.h"
 
 #include "tree.h"
+#include "files.h"
 #include "input.h"
 #include "callin.h"
 #include "cint.h"
 #include "dict.h"
 #include "events.h"
-#include "gstpub.h"
 #include "heap.h"
 #include "lex.h"
 #include "gst-parse.h"
-#include "lib.h"
 #include "oop.h"
 #include "byte.h"
 #include "sym.h"
