@@ -57,8 +57,14 @@
 # include <readline/history.h>
 #endif
 
-#define EMACS_PROCESS_MARKER	'\001'	/* ^A as marker -- random
-					   choice */
+typedef struct gst_file_segment
+{
+  OBJ_HEADER;
+  OOP fileName;
+  OOP startPos;
+  OOP length;
+}
+ *gst_file_segment;
 
 typedef struct string_stream
 {
@@ -96,7 +102,7 @@ typedef struct input_stream
 
   int line;
   int column;
-  mst_Boolean prompt;
+  const char *prompt;
 
   OOP fileNameOOP;		/* the full path name for file */
   const char *fileName;
@@ -138,10 +144,8 @@ static int poll_and_read (int fd, char *buf, int n);
 static int change_str = -1;
 
 
-/* If true, the normal execution information is supressed, and the
-   prompt is emitted with a special marker character ahead of it to
-   let the process filter know that the execution has completed.  */
-mst_Boolean _gst_emacs_process = false;
+/* If true, readline is disabled.  */
+mst_Boolean _gst_no_tty = false;
 
 /* >= 1 if completions are enabled, < 1 if they are not.  Available
    for completeness even if Readline is not used.  */
@@ -220,6 +224,9 @@ _gst_pop_stream (mst_Boolean closeIt)
       xfree (stream->st_str.strBase);
       break;
 
+#ifdef HAVE_READLINE
+    case STREAM_READLINE:
+#endif /* HAVE_READLINE */
     case STREAM_OOP:
       xfree (stream->st_oop.buf);
       _gst_unregister_oop (stream->st_oop.oop);
@@ -230,16 +237,6 @@ _gst_pop_stream (mst_Boolean closeIt)
       if (closeIt)
         close (stream->st_file.fd);
       break;
-
-#ifdef HAVE_READLINE
-    case STREAM_READLINE:
-      if (stream->st_str.strBase)
-	{
-	  xfree (stream->st_str.strBase);
-	  stream->st_str.strBase = NULL;
-	}
-      break;
-#endif /* HAVE_READLINE */
     }
 
   xfree (stream);
@@ -257,7 +254,6 @@ _gst_push_unix_file (int fd,
   newStream->st_file.ptr = newStream->st_file.buf;
   newStream->st_file.end = newStream->st_file.buf;
   newStream->fileName = fileName;
-  newStream->prompt = isatty (fd);
   newStream->fileOffset = lseek (fd, 0, SEEK_CUR);
 }
 
@@ -271,17 +267,8 @@ _gst_push_stream_oop (OOP oop)
   newStream->st_oop.buf = NULL;
   newStream->st_oop.ptr = NULL;
   newStream->st_oop.end = NULL;
-  if (is_a_kind_of (OOP_CLASS (oop), _gst_file_descriptor_class))
-    {
-      gst_file_stream fileStream = (gst_file_stream) OOP_TO_OBJ (oop);
-      newStream->fileNameOOP = fileStream->name;
-      in_stream->fileOffset = lseek (TO_INT (fileStream->file), 0, SEEK_CUR);
-      _gst_register_oop (newStream->fileNameOOP);
-    }
-  else
-    newStream->fileName = "a Smalltalk Stream";
+  newStream->fileName = "a Smalltalk Stream";
 
-  newStream->prompt = false;
   _gst_register_oop (oop);
 }
 
@@ -295,7 +282,6 @@ _gst_push_smalltalk_string (OOP stringOOP)
   newStream->st_str.strBase = (char *) _gst_to_cstring (stringOOP);
   newStream->st_str.str = newStream->st_str.strBase;
   newStream->fileName = "a Smalltalk string";
-  newStream->prompt = false;
 }
 
 void
@@ -308,7 +294,6 @@ _gst_push_cstring (const char *string)
   newStream->st_str.strBase = xstrdup (string);
   newStream->st_str.str = newStream->st_str.strBase;
   newStream->fileName = "a C string";
-  newStream->prompt = false;
 }
 
 void
@@ -317,7 +302,7 @@ _gst_push_stdin_string (void)
 #ifdef HAVE_READLINE
   input_stream newStream;
 
-  if (_gst_emacs_process || !isatty (0))
+  if (_gst_no_tty)
     {
 #endif
       _gst_push_unix_file (0, "stdin");
@@ -329,11 +314,11 @@ _gst_push_stdin_string (void)
     _gst_add_all_symbol_completions ();
 
   newStream = push_new_stream (STREAM_READLINE);
-
-  newStream->st_str.strBase = NULL;	/* force readline() but no free() */
-  newStream->st_str.str = NULL;
+  newStream->fileOffset = 0;
+  newStream->st_oop.buf = NULL;
+  newStream->st_oop.ptr = NULL;
+  newStream->st_oop.end = NULL;
   newStream->fileName = "stdin";	/* that's where we get input from */
-  newStream->prompt = true;
 #endif
 }
 
@@ -350,6 +335,7 @@ push_new_stream (stream_type type)
   newStream->fileOffset = -1;
   newStream->type = type;
   newStream->fileName = NULL;
+  newStream->prompt = NULL;
   newStream->fileNameOOP = _gst_nil_oop;
   newStream->prevStream = in_stream;
   in_stream = newStream;
@@ -359,19 +345,44 @@ push_new_stream (stream_type type)
 
 void
 _gst_set_stream_info (int line,
-		      const char *fileName,
+		      OOP fileNameOOP,
 		      int fileOffset)
 {
   in_stream->line = line;
   in_stream->column = 0;
-  if (fileName)
+  if (!IS_NIL (fileNameOOP))
     {
-      in_stream->fileName = fileName;
-      in_stream->fileNameOOP = _gst_nil_oop;
+      in_stream->fileName = _gst_to_cstring (fileNameOOP);
+      _gst_register_oop (in_stream->fileNameOOP);
     }
+
+  in_stream->fileNameOOP = fileNameOOP;
   in_stream->fileOffset = fileOffset;
 }
  
+void
+refill_stream (input_stream stream, char *buf, int new_line)
+{
+  size_t old_size = stream->st_oop.ptr - stream->st_oop.buf;
+  size_t size = old_size + strlen (buf);
+
+  /* Leave space for the '\0' at the end.  */
+  stream->st_oop.buf = xrealloc (stream->st_oop.buf, size + new_line + 1);
+  stream->st_oop.ptr = stream->st_oop.buf + old_size;
+  stream->st_oop.end = stream->st_oop.buf + size + new_line;
+
+  memcpy (stream->st_oop.ptr, buf, size - old_size);
+  if (new_line)
+    {
+      stream->st_oop.ptr[size - old_size] = '\n';
+      stream->st_oop.ptr[size - old_size + 1] = '\0';
+    }
+  else
+    stream->st_oop.ptr[size - old_size] = '\0';
+
+  free (buf);
+}
+
 int
 my_getc (input_stream stream)
 {
@@ -393,22 +404,11 @@ my_getc (input_stream stream)
       if (stream->st_oop.ptr == stream->st_oop.end)
 	{
 	  char *buf;
-	  int old_size;
-	  int size;
 	  _gst_msg_sendf(&buf, "%s %o nextHunk", stream->st_oop.oop);
 	  if (!buf || !*buf)
 	    return EOF;
 
-	  old_size = stream->st_oop.ptr - stream->st_oop.buf;
-	  size = old_size + strlen (buf);
-
-	  /* Leave space for the '\0' at the end.  */
-	  stream->st_oop.buf = xrealloc (stream->st_oop.buf, size + 1);
-	  stream->st_oop.ptr = stream->st_oop.buf + old_size;
-	  stream->st_oop.end = stream->st_oop.buf + size;
-
-	  memcpy (stream->st_oop.ptr, buf, size - old_size + 1);
-	  free (buf);
+	  refill_stream (stream, buf, false);
 	}
 
       return (unsigned char) *stream->st_oop.ptr++;
@@ -416,10 +416,7 @@ my_getc (input_stream stream)
     case STREAM_FILE:
       if (in_stream->column == 0 && in_stream->prompt)
 	{
-	  if (_gst_emacs_process)
-	    printf ("%c", EMACS_PROCESS_MARKER);
-
-	  printf ("st> ");
+	  printf ("%s", in_stream->prompt);
 	  fflush(stdout);
 	}
 
@@ -440,55 +437,20 @@ my_getc (input_stream stream)
 
 #ifdef HAVE_READLINE
     case STREAM_READLINE:
-      {
-	char *r_line;
-	int r_len;
+      /* Refill the buffer...  */
+      if (stream->st_oop.ptr == stream->st_oop.end)
+	{
+	  char *buf = readline (in_stream->prompt
+				? (char *) in_stream->prompt
+				: (char *) "");
+	  if (!buf)
+	    return EOF;
 
-	if (stream->st_str.strBase)
-	  {
-	    ic = (unsigned char) *stream->st_str.str++;
-	    if (ic)
-	      return (ic);
+	  add_history (buf);
+	  refill_stream (stream, buf, true);
+	}
 
-	    /* If null, read a new line */
-	  }
-
-	if (stream->st_str.strBase)
-	  {
-	    xfree (stream->st_str.strBase);
-	    stream->st_str.strBase = NULL;
-	  }
-	r_line = readline ((char *) "st> ");
-	if (!r_line)
-	  {
-	    /* return value of NULL indicates EOF */
-	    return (EOF);
-	  }
-	if (*r_line)
-	  {
-	    /* add only non-empty lines.  */
-	    add_history (r_line);
-	  }
-
-	/* tack on the newline, not returned by readline() */
-	r_len = strlen (r_line);
-	r_line = xrealloc (r_line, (unsigned) (r_len + 2));
-	if (!r_line)
-	  {
-	    _gst_errorf ("Out of memory reallocating linebuffer space");
-	    stream->st_str.str = stream->st_str.strBase = NULL;
-	    ic = '\n';		/* return a newline ...  */
-	  }
-	else
-	  {
-	    r_line[r_len] = '\n';
-	    r_line[r_len + 1] = '\0';
-	    stream->st_str.str = stream->st_str.strBase = r_line;
-	    ic = (unsigned char) *stream->st_str.str++;
-	  }
-
-	break;
-      }
+      return (unsigned char) *stream->st_oop.ptr++;
 #endif /* HAVE_READLINE */
 
     default:
@@ -502,7 +464,7 @@ my_getc (input_stream stream)
 mst_Boolean
 _gst_get_cur_stream_prompt (void)
 {
-  return !_gst_emacs_process && in_stream && in_stream->prompt;
+  return in_stream && in_stream->prompt;
 }
 
 stream_type
@@ -516,33 +478,73 @@ _gst_get_cur_stream_type (void)
 }
 
 OOP
-_gst_get_cur_string (void)
+_gst_get_source_string (off_t startPos, off_t endPos)
 {
+  char *p;
   OOP result;
   int size;
 
   if (!in_stream)
     return (_gst_nil_oop);
 
+  if (startPos != -1 && !_gst_get_cur_stream_prompt ())
+    {
+      OOP fileName;
+      gst_file_segment fileSegment;
+      inc_ptr incPtr;
+
+      incPtr = INC_SAVE_POINTER ();
+      fileName = _gst_get_cur_file_name ();
+      INC_ADD_OOP (fileName);
+
+      fileSegment = (gst_file_segment) new_instance (_gst_file_segment_class,
+                                                     &result);
+
+      fileSegment->fileName = fileName;
+      fileSegment->startPos = from_c_int_64 (startPos);
+      fileSegment->length = from_c_int_64 (endPos - startPos);
+
+      assert (to_c_int_64 (fileSegment->length) >= 0);
+      INC_RESTORE_POINTER (incPtr);
+      return (result);
+    }
+
   switch (in_stream->type)
     {
     case STREAM_STRING:
-      return (_gst_string_new (in_stream->st_str.strBase));
+      p = in_stream->st_str.strBase;
+      break;
 
+#ifdef HAVE_READLINE
+    case STREAM_READLINE:
+#endif /* HAVE_READLINE */
     case STREAM_OOP:
-      result = _gst_string_new (in_stream->st_oop.buf);
-
-      /* Copy back to the beginning of the buffer to save memory.  */
-      size = in_stream->st_oop.end - in_stream->st_oop.ptr + 1;
-      memmove (in_stream->st_oop.buf, in_stream->st_oop.ptr, size);
-      in_stream->fileOffset += in_stream->st_oop.ptr - in_stream->st_oop.buf;
-      in_stream->st_oop.ptr = in_stream->st_oop.buf;
-      in_stream->st_oop.end = in_stream->st_oop.buf + size;
-      return (result);
+      p = in_stream->st_oop.buf;
+      break;
 
     default:
       return (_gst_nil_oop);
     }
+
+   if (startPos == -1) 
+    result = _gst_string_new (p);
+  else
+    result = _gst_counted_string_new (p + (startPos - in_stream->fileOffset),
+				      endPos - startPos);
+
+  if (in_stream->type != STREAM_STRING)
+    {
+      /* Copy back to the beginning of the buffer to save memory.  */
+      size = in_stream->st_oop.end - in_stream->st_oop.ptr;
+      if (size)
+	memmove (in_stream->st_oop.buf, in_stream->st_oop.ptr, size);
+      in_stream->st_oop.buf[size] = 0;
+      in_stream->fileOffset += in_stream->st_oop.ptr - in_stream->st_oop.buf;
+      in_stream->st_oop.ptr = in_stream->st_oop.buf;
+      in_stream->st_oop.end = in_stream->st_oop.buf + size;
+    }
+
+  return result;
 }
 
 OOP
@@ -701,11 +703,8 @@ line_stamp (int line)
 	{
 	  if (in_stream->fileName)
 	    fprintf (stderr, "%s:", in_stream->fileName);
-	  else if (in_stream->fileNameOOP != _gst_nil_oop)
-	    {
-	      char *s = _gst_to_cstring (in_stream->fileNameOOP);
-	      fprintf (stderr, "%s:", s);
-	    }
+	  else if (!IS_NIL (in_stream->fileNameOOP))
+	    fprintf (stderr, "%#O:", in_stream->fileNameOOP);
 
 	  fprintf (stderr, "%d: ", line);
 	}
@@ -818,6 +817,58 @@ poll_and_read (int fd, char *buf, int n)
     return -1;
 }
 
+void
+_gst_process_stdin (const char *prompt)
+{
+  if (_gst_verbosity == 3 || isatty (0))
+    {
+      printf ("GNU Smalltalk ready\n\n");
+      fflush (stdout);
+    }
+
+  _gst_non_interactive = false;
+  _gst_push_stdin_string ();
+  if (isatty (0))
+    in_stream->prompt = prompt;
+  _gst_parse_stream (false);
+  _gst_pop_stream (true);
+  _gst_non_interactive = true;
+}
+
+mst_Boolean
+_gst_process_file (const char *fileName, enum gst_file_dir dir)
+{
+  enum undeclared_strategy old;
+  int fd;
+  char *f;
+
+  f = _gst_find_file (fileName, dir);
+  if (!f)
+    {
+      errno = ENOENT;
+      return false;
+    }
+
+  errno = 0;
+  fd = _gst_open_file (f, "r");
+  if (fd != -1)
+    {
+      if (_gst_verbosity == 3)
+	printf ("Processing %s\n", f);
+
+      old = _gst_set_undeclared (UNDECLARED_GLOBALS);
+      _gst_push_unix_file (fd, f);
+      _gst_parse_stream (false);
+      _gst_pop_stream (true);
+      _gst_set_undeclared (old);
+      errno = 0;
+    }
+
+  xfree (f);
+  return (fd != -1);
+}
+
+
 #ifdef HAVE_READLINE
 /* Find apostrophes and double them */
 char *
