@@ -66,10 +66,8 @@ typedef enum
 }
 stream_type;
 
-/* If true, the normal execution information is supressed, and the
-   prompt is emitted with a special marker character ahead of it to
-   let the process filter know that the execution has completed.  */
-extern mst_Boolean _gst_emacs_process 
+/* If true, readline is suppressed.  */
+extern mst_Boolean _gst_no_tty 
   ATTRIBUTE_HIDDEN;
 
 /* Pass file descriptor FD, printed as file name FILENAME, to the
@@ -117,7 +115,8 @@ extern void _gst_pop_stream (mst_Boolean closeIt)
    a subsection of a real file via a temporary file what the real
    source of the text is.  */
 extern void _gst_set_stream_info (int line,
-				  const char *fileName,
+				  OOP fileOOP,
+				  OOP fileNameOOP,
 				  int fileOffset) 
   ATTRIBUTE_HIDDEN;
 
@@ -176,12 +175,6 @@ extern stream_type _gst_get_cur_stream_type (void)
   ATTRIBUTE_PURE 
   ATTRIBUTE_HIDDEN;
 
-/* Return the name of the topmost stream in the stack if it is of type
-   STREAM_FILE; nil otherwise.  */
-extern OOP _gst_get_cur_file_name (void)
-  ATTRIBUTE_PURE 
-  ATTRIBUTE_HIDDEN;
-
 /* Returns an OOP containing the string of the topmost stream if it is
    a STREAM_STRING, STREAM_OOP or STREAM_READLINE, or if it has a filename;
    nil otherwise.  */
@@ -212,8 +205,19 @@ extern void _gst_enable_completion (void)
 extern void _gst_disable_completion (void) 
   ATTRIBUTE_HIDDEN;
 
+#endif /* HAVE_READLINE */
 
-#endif /* GST_INPUT_H */
+
+/* Parse the Smalltalk source code read from stdin, showing the
+   PROMPT that is passed.  */
+extern void _gst_process_stdin (const char *prompt)
+  ATTRIBUTE_HIDDEN;
+
+/* Parse the Smalltalk source code read from file FILE found within
+   the search path DIR.  */
+extern mst_Boolean _gst_process_file (const char *fileName,
+				      enum gst_file_dir dir)
+  ATTRIBUTE_HIDDEN;
 
 
 #endif
