@@ -10,7 +10,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -351,10 +351,13 @@ static void sleep_process (OOP processOOP);
    processes for PROCESSOOP's priority.  Return PROCESSOOP.  */
 static OOP activate_process (OOP processOOP);
 
+/* Restore the virtual machine's state from the ContextPart OOP.  */
+static void resume_suspended_context (OOP oop);
+
 /* Save the virtual machine's state into the suspended Process and
    ContextPart objects, and load them from NEWPROCESS and from
    NEWPROCESS's suspendedContext.  The Processor (the only instance
-   of ProcessorScheduler is also updated accordingly.  */
+   of ProcessorScheduler) is also updated accordingly.  */
 static void change_process_context (OOP newProcess);
 
 /* Mark the semaphores attached to the process system (asynchronous
@@ -421,9 +424,10 @@ static inline OOP create_args_array (int numArgs);
    the arguments in the block context, which have been copied out of
    the caller's context. 
 
-   On failure return true, on success (i.e. if NUMARGS matches what
-   the BlockClosure says) return false.  */
-static mst_Boolean send_block_value (int numArgs);
+   The block should accept between NUMARGS - CULL_UP_TO and
+   NUMARGS arguments.  If this is not true (failure) return true;
+   on success return false.  */
+static mst_Boolean send_block_value (int numArgs, int cull_up_to);
 
 /* This is a kind of simplified _gst_send_message_internal that,
    instead of setting up a context for a particular receiver, stores
@@ -471,11 +475,6 @@ static inline void prepare_context (gst_context_part context,
    status (ip, sp, _gst_this_method, _gst_self, ...).  */
 static void unwind_context (void);
 
-/* Used to help minimize the number of primitives used to control the
-   various debugging flags, this routine maps the variable's INDEX to the
-   address of a boolean debug flag, which it returns.  */
-static inline int *bool_addr_index (int index) ATTRIBUTE_PURE;
-
 /* Check whether it is true that sending SENDSELECTOR to RECEIVER
    accepts NUMARGS arguments.  Note that the RECEIVER is only used to
    do a quick check in the method cache before examining the selector
@@ -1321,7 +1320,6 @@ _gst_make_block_closure (OOP blockOOP)
 void
 change_process_context (OOP newProcess)
 {
-  gst_method_context thisContext;
   OOP processOOP;
   gst_process process;
   gst_processor_scheduler processor;
@@ -1352,7 +1350,7 @@ change_process_context (OOP newProcess)
       disable_preemption = !IS_NIL (process->interrupts) &&
 		           TO_INT (process->interrupts) < 0;
 
-      _gst_this_context_oop = process->suspendedContext;
+      resume_suspended_context (process->suspendedContext);
 
       /* Interrupt-enabling cannot be controlled globally from Smalltalk,
          but only on a per-Process basis.  You might think that this leaves
@@ -1377,8 +1375,15 @@ change_process_context (OOP newProcess)
             _gst_enable_interrupts();
 	}
     }
+}
 
-  thisContext = (gst_method_context) OOP_TO_OBJ (_gst_this_context_oop);
+void
+resume_suspended_context (OOP oop)
+{
+  gst_method_context thisContext;
+
+  _gst_this_context_oop = oop;
+  thisContext = (gst_method_context) OOP_TO_OBJ (oop);
   SET_THIS_METHOD (thisContext->method, GET_CONTEXT_IP (thisContext));
   sp = thisContext->contextStack + TO_INT (thisContext->spOffset);
 
@@ -1388,6 +1393,7 @@ change_process_context (OOP newProcess)
 
   _gst_temporaries = thisContext->contextStack;
   _gst_self = thisContext->receiver;
+  free_lifo_context = lifo_contexts;
 }
 
 
@@ -1457,8 +1463,8 @@ add_first_link (OOP semaphoreOOP,
     sem->lastLink = processOOP;
 }
 
-void
-suspend_process (OOP processOOP)
+static void
+remove_process_from_list (OOP processOOP)
 {
   gst_semaphore sem;
   gst_process process, lastProcess;
@@ -1499,6 +1505,12 @@ suspend_process (OOP processOOP)
     }
 
   process->nextLink = _gst_nil_oop;
+}
+
+void
+suspend_process (OOP processOOP)
+{
+  remove_process_from_list (processOOP);
   if (get_scheduled_process() == processOOP)
     ACTIVE_PROCESS_YIELD ();
 }
@@ -1680,11 +1692,25 @@ resume_process (OOP processOOP,
 
   /* 2002-19-12: tried get_active_process instead of get_scheduled_process.  */
   activeOOP = get_active_process ();
-  if (processOOP == activeOOP)
-    return (true);
-
   active = (gst_process) OOP_TO_OBJ (activeOOP);
   process = (gst_process) OOP_TO_OBJ (processOOP);
+  priority = TO_INT (process->priority);
+
+  /* As a special exception, don't preempt a process that has disabled
+     interrupts. ### this behavior is currently disabled.  */
+  ints_enabled = IS_NIL (active->interrupts)
+	         || TO_INT(active->interrupts) <= 0;
+
+  /* resume_process is also used when changing the priority of a ready/active
+     process.  In this case, first remove the process from its current list.  */
+  if (processOOP == activeOOP)
+    {
+      assert (!alwaysPreempt);
+      remove_process_from_list (processOOP);
+    }
+  else if (priority >= TO_INT (active->priority) /* && ints_enabled */ )
+    alwaysPreempt = true;
+
   if (IS_NIL (processOOP) || is_process_terminating (processOOP))
     /* The process was terminated - nothing to resume, fail */
     return (false);
@@ -1696,17 +1722,10 @@ resume_process (OOP processOOP,
       return (true);
     }
 
-  /* As a special exception, don't preempt a process that has disabled
-     interrupts. ### this behavior is currently disabled.  */
-  ints_enabled = IS_NIL (active->interrupts)
-	         || TO_INT(active->interrupts) <= 0;
-
-  priority = TO_INT (process->priority);
   processLists = GET_PROCESS_LISTS ();
   processList = ARRAY_AT (processLists, priority);
 
-  if ((priority >= TO_INT (active->priority) /* && ints_enabled */ )
-      || alwaysPreempt)
+  if (alwaysPreempt)
     {
       /* We're resuming a process with a *equal or higher* priority, so sleep
          the current one and activate the new one */
@@ -1714,11 +1733,13 @@ resume_process (OOP processOOP,
       activate_process (processOOP);
     }
   else
-    /* this process has a lower priority than the active one, so the
-       policy is that it doesn't preempt the currently running one.
-       Anyway, it must be the first in its priority queue - so don't
-       put it to sleep.  */
-    add_first_link (processList, processOOP);
+    {
+      /* this process has a lower priority than the active one, so the
+         policy is that it doesn't preempt the currently running one.
+         Anyway, it must be the first in its priority queue - so don't
+         put it to sleep.  */
+      add_first_link (processList, processOOP);
+    }
 
   return (true);
 }
@@ -2025,25 +2046,66 @@ create_callin_process (OOP contextOOP)
   return (initialProcessOOP);
 }
 
-int *
-bool_addr_index (int index)
+int
+_gst_get_var (enum gst_var_index index)
 {
   switch (index)
     {
-    case 0:
-      return (&_gst_declare_tracing);
-    case 1:
-      return (&_gst_execution_tracing);
-    case 2:
-      return (&verbose_exec_tracing);
-    case 3:
-      return (&_gst_gc_message);
+    case GST_DECLARE_TRACING:
+      return (_gst_declare_tracing);
+    case GST_EXECUTION_TRACING:
+      return (_gst_execution_tracing);
+    case GST_EXECUTION_TRACING_VERBOSE:
+      return (verbose_exec_tracing);
+    case GST_GC_MESSAGE:
+      return (_gst_gc_message);
+    case GST_VERBOSITY:
+      return (_gst_verbosity);
+    case GST_MAKE_CORE_FILE:
+      return (_gst_make_core_file);
+    case GST_REGRESSION_TESTING:
+      return (_gst_regression_testing);
     default:
-      return (NULL);		/* index out of range, signal the error 
-				 */
+      return (-1);
     }
 }
 
+int
+_gst_set_var (enum gst_var_index index, int value)
+{
+  int old = _gst_get_var (index);
+  if (value < 0)
+    return -1;
+
+  switch (index)
+    {
+    case GST_DECLARE_TRACING:
+      _gst_declare_tracing = value;
+      break;
+    case GST_EXECUTION_TRACING:
+      _gst_execution_tracing = value;
+      break;
+    case GST_EXECUTION_TRACING_VERBOSE:
+      verbose_exec_tracing = value;
+      break;
+    case GST_GC_MESSAGE:
+      _gst_gc_message = value;
+      break;
+    case GST_VERBOSITY:
+      _gst_verbosity = value;
+      break;
+    case GST_MAKE_CORE_FILE:
+      _gst_make_core_file = value;
+      break;
+    case GST_REGRESSION_TESTING:
+      _gst_regression_testing = true;
+      break;
+    default:
+      return (-1);
+    }
+
+  return old;
+}
 
 
 void
@@ -2072,7 +2134,8 @@ OOP
 _gst_prepare_execution_environment (void)
 {
   gst_method_context dummyContext;
-  OOP dummyContextOOP;
+  OOP dummyContextOOP, processOOP;
+  inc_ptr inc = INC_SAVE_POINTER ();
 
   empty_context_stack ();
   dummyContext = alloc_stack_context (4);
@@ -2095,7 +2158,12 @@ _gst_prepare_execution_environment (void)
   dummyContextOOP = alloc_oop (dummyContext,
 			       _gst_mem.active_flag | F_POOLED | F_CONTEXT);
 
-  return (create_callin_process (dummyContextOOP));
+
+  INC_ADD_OOP (dummyContextOOP);
+  processOOP = create_callin_process (dummyContextOOP);
+
+  INC_RESTORE_POINTER (inc);
+  return (processOOP);
 }
 
 OOP
@@ -2496,7 +2564,7 @@ _gst_show_backtrace (void)
 	      if (IS_NIL(context->parentContext))
 	        printf ("<bottom>\n");
 	      else
-	        printf ("<call-in>\n");
+	        printf ("<unwind point>\n");
 	      continue;
 	    }
 
