@@ -56,6 +56,16 @@
 #ifndef GST_OOP_H
 #define GST_OOP_H
 
+/* Define this flag to disable blacking of grey pages (that is, the
+   entire oldspace is scanned to look for reachable newspace objects).
+   This is also necessary to run valgrind on GNU Smalltalk.  */
+/* #define NO_SIGSEGV_HANDLING */
+
+/* ... but always define it if libsigsegv does not support this platform.  */
+#if !defined HAVE_SIGSEGV_RECOVERY || !(HAVE_SIGSEGV_RECOVERY-0)
+#define NO_SIGSEGV_HANDLING
+#endif
+
 #define NUM_CHAR_OBJECTS	256
 #define NUM_BUILTIN_OBJECTS	3
 #define FIRST_OOP_INDEX		(-NUM_CHAR_OBJECTS-NUM_BUILTIN_OBJECTS)
@@ -217,6 +227,9 @@ struct memory_space
   /* The active survivor space */
   struct surv_space *active_half;
 
+  /* The beginning and end of the area mmap-ed directly from the image.  */
+  OOP *loaded_base, *loaded_end;
+
   /* The OOP flag corresponding to the active survivor space */
   int active_flag;
 
