@@ -11,7 +11,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -139,6 +139,9 @@
 #define PATH_MAX  1024		/* max length of a file and path */
 #endif
 
+#ifndef MAXSYMLINKS
+#define MAXSYMLINKS 5
+#endif
 
 /* Yield A - B, measured in seconds.
    This function is copied from the GNU C Library.  */
@@ -801,6 +804,7 @@ _gst_get_full_file_name (const char *name)
 
 #if defined WIN32 && !defined __CYGWIN__
   {
+    struct stat st;
     DWORD retval = GetFullPathNameA (name, path_max, rpath, NULL);
     if (retval > path_max)
       {
@@ -943,16 +947,44 @@ error:
 
 
 
-time_t
-_gst_get_file_modify_time (const char *fileName)
+mst_Boolean
+_gst_file_is_newer (const char *file1, const char *file2)
 {
-  struct stat st;
+  static char *prev_file1;
+  static struct stat st1;
+  struct stat st2;
 
-  if (stat (fileName, &st) < 0)
-    return (0);
+  if (!prev_file1 || strcmp (file1, prev_file1))
+    {
+      if (prev_file1)
+	xfree (prev_file1);
+      prev_file1 = xstrdup (file1);
 
-  else
-    return (_gst_adjust_time_zone (st.st_mtime));
+      if (!_gst_file_is_readable (file1))
+        return false;
+      if (stat (file1, &st1) < 0)
+	return false;
+    }
+
+  if (!_gst_file_is_readable (file2))
+    return true;
+  if (stat (file2, &st2) < 0)
+    return true;
+
+  if (st1.st_mtime != st2.st_mtime)
+    return st1.st_mtime > st2.st_mtime;
+
+  /* 15 years have passed and nothing seems to have changed.  */
+#if defined HAVE_STRUCT_STAT_ST_MTIMENSEC
+  return st1.st_mtimensec >= st2.st_mtimensec;
+#elif defined HAVE_STRUCT_STAT_ST_MTIM_TV_NSEC
+  return st1.st_mtim.tv_nsec >= st2.st_mtim.tv_nsec;
+#elif defined HAVE_STRUCT_STAT_ST_MTIMESPEC_TV_NSEC
+  return st1.st_mtimespec.tv_nsec >= st2.st_mtimespec.tv_nsec;
+#else
+  /* Say that the image file is newer.  */
+  return true;
+#endif
 }
 
 
@@ -979,6 +1011,9 @@ _gst_file_is_executable (const char *fileName)
 static int executable_fd = -1;
 #endif
 
+/* The path to the executable, derived from argv[0].  */
+const char *_gst_executable_path = NULL;
+
 /* Tests whether a given pathname may belong to the executable.  */
 static mst_Boolean
 maybe_executable (const char *filename)
@@ -1012,8 +1047,8 @@ maybe_executable (const char *filename)
    Return NULL if unknown.  Guaranteed to work on Linux and Win32, Mac OS X.
    Likely to work on the other Unixes (maybe except BeOS), under most
    conditions.  */
-char *
-_gst_find_executable (const char *argv0)
+static char *
+find_executable (const char *argv0)
 {
 #if defined WIN32
   char location[MAX_PATH];
@@ -1126,6 +1161,46 @@ _gst_find_executable (const char *argv0)
 #endif
 }
 
+void
+_gst_set_executable_path (const char *argv0)
+{
+  _gst_executable_path = find_executable (argv0);
+}
+
+char *
+_gst_relocate_path (const char *path)
+{
+  const char *p;
+  char *s;
+
+  /* Detect absolute paths.  */
+#if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
+  if ((path[0] && path[1] == ':')
+      || path[0] == '/' || path[0] == '\\')
+    return xstrdup (path);
+#else
+  if (path[0] == '/')
+    return xstrdup (path);
+#endif
+
+  /* Remove filename from executable path.  */
+  p = _gst_executable_path + strlen (_gst_executable_path);
+  do
+    --p;
+  while (p >= _gst_executable_path
+         && *p != '/'
+#if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
+	 && *p != '\\'
+#endif
+	);
+  p++;
+
+  /* Now p points just past the last separator (if any).  */
+  s = alloca (p - _gst_executable_path + strlen (path) + 1);
+  sprintf (s, "%.*s%s", p - _gst_executable_path, _gst_executable_path, path);
+  return _gst_get_full_file_name (s);
+}
+
 
 /* Code to use PTY's did not work on Mac OS.  I'm keeping the Unix code,
    but it should not be used.  */
@@ -1640,7 +1715,7 @@ _gst_open_file (const char *filename,
       if (*mode == '\0')
 	break;
       else if (*mode == '+')
-	access = O_RDWR;
+        create = 1, access = O_RDWR;
       else if (*mode == 'x')
 	oflags |= O_EXCL;
     }
