@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -81,6 +81,9 @@
 
 typedef RETSIGTYPE (*SigHandler) ();
 
+/* The path to the executable.  */
+extern const char *_gst_executable_path;
+
 /* Saves and returns the current state of the software interrupt
    system.  Disables all interrupts.  */
 extern void _gst_disable_interrupts (void)
@@ -130,12 +133,16 @@ extern unsigned _gst_get_milli_time (void)
 extern time_t _gst_get_time (void)
   ATTRIBUTE_HIDDEN;
 
-/* Returns the time the file FILENAME was last modified.  On UNIX
-   machines, this is the number of seconds since midnight Jan 1 1970
-   GMT.  On other platforms/environments, it's probably not important
-   exactly what it returns as long as it's unites consistent with
-   other accesses that client code may do to the file system.  */
-extern time_t _gst_get_file_modify_time (const char *fileName)
+/* Returns whether FILE1 is newer (or last modified at the same time as)
+   FILE2.  Returns true if FILE2 is not readable, false if FILE1 is not
+   readable.  */
+extern mst_Boolean _gst_file_is_newer (const char *file1, const char *file2)
+  ATTRIBUTE_HIDDEN;
+
+/* Sets the time when FILENAME was last modified.  The times are in
+   seconds, relative to 1 Jan 2000.  */
+extern int _gst_set_file_access_times (const char *name, long new_atime,
+				       long new_mtime)
   ATTRIBUTE_HIDDEN;
 
 /* Converts the given time (expressed in seconds since midnight Jan 1,
@@ -180,6 +187,15 @@ extern mst_Boolean _gst_file_is_writeable (const char *fileName)
 extern mst_Boolean _gst_file_is_executable (const char *fileName)
   ATTRIBUTE_HIDDEN;
 
+/* Return a path to the executable given argv[0].  */
+extern void _gst_set_executable_path (const char *argv0)
+  ATTRIBUTE_HIDDEN;
+
+/* Return the absolute path for PATH, interpreted relative to the
+   executable.  */
+char *_gst_relocate_path (const char *path)
+  ATTRIBUTE_HIDDEN;
+
 /* Answer true if the file descriptor FD is associated to a pipe
    (it cannot be seeked through).  */
 extern mst_Boolean _gst_is_pipe (int fd)
