@@ -1,5 +1,5 @@
 /* Fault handler information.  MacOSX/i386 version.
-   Copyright (C) 2003-2004, 2006  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 2003-2004, 2006-2007  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,12 @@
 #define SIGSEGV_EXC_STATE_TYPE                      i386_exception_state_t
 #define SIGSEGV_EXC_STATE_FLAVOR                    i386_EXCEPTION_STATE
 #define SIGSEGV_EXC_STATE_COUNT                     i386_EXCEPTION_STATE_COUNT
+#if __DARWIN_UNIX03
+#define SIGSEGV_FAULT_ADDRESS(thr_state,exc_state)  (exc_state).__faultvaddr
+#define SIGSEGV_STACK_POINTER(thr_state)            (thr_state).__esp
+#define SIGSEGV_PROGRAM_COUNTER(thr_state)          (thr_state).__eip
+#else
 #define SIGSEGV_FAULT_ADDRESS(thr_state,exc_state)  (exc_state).faultvaddr
 #define SIGSEGV_STACK_POINTER(thr_state)            (thr_state).esp
 #define SIGSEGV_PROGRAM_COUNTER(thr_state)          (thr_state).eip
+#endif
